/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
package base.gui;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextField;

import main.FunSiPGui;
import main.FunSiPJar;

/**
 * This class displays possible options (general) from the GUI-section
 * of FunSiP. 
 * @author Michiel Van Bel
 *
 */
public class OptionsPanel extends JPanel implements ActionListener{
	
	public static final long serialVersionUID = 112548455l;
	
	private FunSiPGui fspg = null;
	private JFrame parent = null;
	
	//listing of objects related to the use of various options
	private JTextField currentOutputDirectory = null;
	private JButton findCurrentOutputDirectory = null;
	
	public OptionsPanel(FunSiPGui fspg,JFrame parent){
		super();
		this.fspg = fspg;
		this.parent= parent;
		
		createContent();
	}
	
	public void setOptions(){
		if(this.fspg!=null){
			//do the outputdir thingie
			String dir = currentOutputDirectory.getText();
			File f = new File(dir);
			if(f.exists()&&f.isDirectory()){
				if(!dir.endsWith("/")&&!dir.endsWith("\\"))
					dir+="/";
				FunSiPJar.USER_DIR = dir;
			}
			else{
				String message = "Indicated outputdirectory cannot be located.";
				JOptionPane.showMessageDialog(parent,message,"Cannot set options",JOptionPane.ERROR_MESSAGE);
			}
		}
	}
	
	private void setDimensions(Component c,Dimension d){
		c.setPreferredSize(d);
		c.setMinimumSize(d);
		c.setMaximumSize(d);
	}
	
	private void createContent(){
		setDimensions(this,new Dimension(500,200));
		this.setLayout(new GridLayout(0,1));
		
		//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
		//OUTPUT DIRECTORY OPTIONS
		//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
		JPanel currentOutputPanel = new JPanel(new GridLayout(1,3));
		JPanel currentOutputPanel1 = new JPanel();
		JPanel currentOutputPanel2 = new JPanel();
		JPanel currentOutputPanel3 = new JPanel();
		
		JLabel currentOutputLabel = new JLabel("Current output directory");
		currentOutputDirectory = new JTextField(20);
		findCurrentOutputDirectory = new JButton("Locate directory");
		findCurrentOutputDirectory.addActionListener(this);
		currentOutputPanel1.add(currentOutputLabel);
		currentOutputPanel2.add(currentOutputDirectory);
		currentOutputPanel3.add(findCurrentOutputDirectory);
		currentOutputPanel.add(currentOutputPanel1);
		currentOutputPanel.add(currentOutputPanel2);
		currentOutputPanel.add(currentOutputPanel3);
		File cod = new File(FunSiPJar.USER_DIR);
		currentOutputDirectory.setText(cod.getAbsolutePath());
		
		
		JPanel option2Panel = new JPanel();
		JPanel option3Panel = new JPanel();
		
		currentOutputPanel.setBorder(BorderFactory.createEtchedBorder());
		option2Panel.setBorder(BorderFactory.createEtchedBorder());
		option3Panel.setBorder(BorderFactory.createEtchedBorder());
		
		this.add(currentOutputPanel);
		this.add(option2Panel);
		this.add(option3Panel);
	}
	
	
	public void actionPerformed(ActionEvent e){
		if(e.getSource().equals(findCurrentOutputDirectory)){
			JFileChooser chooser = new JFileChooser();
			chooser.setMultiSelectionEnabled(false);
			chooser.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
			int res = chooser.showOpenDialog(this);
			if(res==JFileChooser.APPROVE_OPTION && chooser.getSelectedFile().isDirectory())
				currentOutputDirectory.setText(chooser.getSelectedFile().getPath());
		}
	}
	

}
