/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
package base.gui;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.IOException;
import java.io.Writer;

import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextPane;
import javax.swing.text.Style;
import javax.swing.text.StyleConstants;
import javax.swing.text.StyleContext;
import javax.swing.text.StyledDocument;

import main.FunSiPGui;

public class SprOutputEditorWriter extends JPanel implements Runnable, ActionListener{
	
	public static final long serialVersionUID = 11254658l;
	
	private JTextPane pane = null;
	private Writer writer = null;
	
	
	private FunSiPGui rt = null;
	
	public SprOutputEditorWriter(){
		super(new BorderLayout());
		pane = new JTextPane();
		pane.setEditable(false);
		writer = new TextPaneWriter(pane);
		
		Dimension d = new Dimension(600,500);
		JScrollPane scroll = new JScrollPane(pane);
		scroll.setMinimumSize(d);
		scroll.setMaximumSize(d);
		scroll.setPreferredSize(d);
		
		this.add(scroll,BorderLayout.CENTER);
	}
	
	public void actionPerformed(ActionEvent e){
		if(rt==null)
			return;
	}
	
	public void run(){
		try{
			pane.write(writer);
		}
		catch(Exception exc){
			
		}
	}
	
	public Writer getWriter(){
		return writer;
	}

	public void setRuntimeFunSiP(FunSiPGui fs){
		this.rt = fs;
	}
	
	
	private class TextPaneWriter extends Writer{
		
		//private final JTextPane textpane;
		private final StyledDocument doc;
		
		public TextPaneWriter(final JTextPane textpane){
			//this.textpane = textpane;
			this.doc = textpane.getStyledDocument();
			addStylesToDocument();			
		}
		
		public void addStylesToDocument(){
			Style def = StyleContext.getDefaultStyleContext().getStyle(StyleContext.DEFAULT_STYLE);
			Style regular = doc.addStyle("regular",def);
			
			Style fatal = doc.addStyle("fatal",regular);
			StyleConstants.setForeground(fatal, Color.red);
			
			Style error = doc.addStyle("error", regular);
			StyleConstants.setForeground(error, new Color(200,0,150));
			
			Style warn = doc.addStyle("warn", regular);
			StyleConstants.setForeground(warn, new Color(20,250,50));
			
			Style info = doc.addStyle("info", regular);
			StyleConstants.setForeground(info, Color.BLUE);
			
			Style debug = doc.addStyle("debug", regular);
			StyleConstants.setForeground(debug, Color.BLACK);
			
		}
		
		@Override
		public void flush(){}
		
		@Override
		public void close(){}
		
		@Override
		public void write(char[] cbuf, int off, int len) throws IOException{
			String t = new String(cbuf,off,len);
			String att = null;
			
			if(t.startsWith("FATAL")){
				att = "fatal";
			}
			else if(t.startsWith("ERROR")){
				att = "error";
			}
			else if(t.startsWith("WARN")){
				att = "warn";
			}
			else if(t.startsWith("INFO")){
				att = "info";
			}
			else if(t.startsWith("DEBUG")){
				att = "debug";
			}
			
			try{
				doc.insertString(doc.getLength(),t,doc.getStyle(att));
			}
			catch(Exception exc){
				//do nothing
			}
		}
	}
}
