package main;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;
import java.util.TreeSet;

import util.classificationFeatures.ClassificationFeature;
import util.classificationFeatures.ClassificationFeatureUtils;
import util.classificationFeatures.implementations.PositionalClassificationFeature;
import util.io.FileParser;

public class FunSiPFeatureExplorer {
	
	public static enum POSSIBLE_COMBINATION_TYPES{PRESENT_COMBINATIONS,NOT_PRESENT_COMBINATIONS};

	private List<ClassificationFeature> features = new ArrayList<ClassificationFeature>();
	//outer-->inner : sequence->featuretype->features
	private List<List<List<Double>>> extractedFeatures = new ArrayList<List<List<Double>>>();
	
	public final static double MINIMUM_SUPPORT_FIRST	= 0.98;
	
	public final static int MAX_NUMBER = 100;
	//private double MINIMUM_SUPPORT_LAST 	= 0.50;
	
	public static void main(String[] args){
		if(args.length!=3){
			System.err.println("Need 3 args");
			System.exit(1);
		}
		else{
			new  FunSiPFeatureExplorer(args[0],args[1],Integer.parseInt(args[2]));
		}
	}
	
	public FunSiPFeatureExplorer(String posFileName,String negFileName,int splicesite){
		//define all features.
		features = defineFeatures();
		if(features==null){
			System.err.println("Features not defined");
			return;
		}
		
		System.out.println("READING TRAINING DATA");
		//read the training data.
		List<String> posData = null;
		List<String> negData = null;
		try{
			FileParser parser = new FileParser();
			//posData = parser.extractSequencesFromFile(posFileName,100);
			posData=parser.extractRandomSequencesFromFile(posFileName, 2000);
			negData=parser.extractRandomSequencesFromFile(negFileName, 2000);
		}
		catch(Exception exc){
			exc.printStackTrace();
			return;
		}
		
		System.out.println("Extracting, validating and sorting data for positive data");
		long time1 = System.currentTimeMillis();
		extractFeatures(posData,splicesite);
		List<Combination> pos_combs = doCombinations();
		pos_combs = findDescriptions(pos_combs);
		pos_combs = filter_dubbels(pos_combs);
		pos_combs = filter_splice_locations(pos_combs);
		pos_combs = filter_overlapping(pos_combs);
		//pos_combs = sorting_and_selecting(pos_combs);
		long time2 = System.currentTimeMillis();
		System.out.println("Used time : "+(time2-time1)/1000+" seconds");
		
		System.out.println("Extracting, validating and sorting data for negative data");
		time1 = System.currentTimeMillis();
		extractFeatures(negData,splicesite);
		List<Combination> neg_combs = doCombinations();
		neg_combs = findDescriptions(neg_combs);
		neg_combs = filter_dubbels(neg_combs);
		neg_combs = filter_splice_locations(neg_combs);
		neg_combs = filter_overlapping(neg_combs);
		time2 = System.currentTimeMillis();
		System.out.println("Used time : "+(time2-time1)/1000+" seconds");
		//neg_combs = sorting_and_selecting(neg_combs);
		
		System.out.println("Merging results of positive and negative data");
		time1 = System.currentTimeMillis();
		System.out.println("Size of positives before merging: "+pos_combs.size());
		List<Combination> combs = findBestCombinations(pos_combs,neg_combs);
		System.out.println("Size of negatives after merging: "+combs.size());
		System.out.println("Before min distance selection : "+combs.size());
		combs = do_extra_selection(combs);
		System.out.println("After min distance selection : "+combs.size());
		combs = sorting_and_selecting(combs);
		printToFile(combs);
		time2 = System.currentTimeMillis();
		System.out.println("Used time :"+(time2-time1)/1000+" seconds");
		showResultingCombinations(combs);

	}
	
	public List<Combination> do_extra_selection(List<Combination>combs){
		List<Combination> result = new ArrayList<Combination>();
		for(int i=0;i<combs.size();i++){
			Combination comb = combs.get(i);
			String[] split1 = comb.description_1.split(" ");
			String[] split2 = comb.description_2.split(" ");
			int l1 = Integer.parseInt(split1[2]);
			int l2 = Integer.parseInt(split2[2]);
			int distance = (int)Math.abs(l1-l2);
			if(split1[3].equals(split2[3]) && distance<=2){
				//do nothing
			}
			else{
				result.add(comb);
			}
		}
		return result;
	}
	
	public List<Combination> findBestCombinations(List<Combination> posCombs,List<Combination>negCombs){
		List<Combination> result = new ArrayList<Combination>();
		for(int i=0;i<posCombs.size();i++){
			Combination positive = posCombs.get(i);
			boolean test = true;
			for(int j=0;j<negCombs.size();j++){
				Combination negative = negCombs.get(j);
				if(positive.equals(negative)){
					if(Math.abs(positive.support-negative.support)<0.9){
						test = false;
						break;
					}
				}
			}
			if(test){
				result.add(positive);
			}
		}
		
		return result;
	}
	
	public void printToFile(List<Combination> combs){
		try{
			BufferedWriter writer = new BufferedWriter(new FileWriter(new File("./negative_filter.txt")));
			writer.write("TYPE\t"+POSSIBLE_COMBINATION_TYPES.NOT_PRESENT_COMBINATIONS+"\n");
			for(int i=0;i<features.size();i++)
				writer.write("CLASSIFICATION_FEATURE\t"+features.get(i)+"\n");
			for(int i=0;i<combs.size();i++){
				Combination comb = combs.get(i);
				writer.write("COMBINATION\t"+comb.first_feature+" "+comb.first_position+" "+
						comb.second_feature+" "+comb.second_position+"\n");
			}
			
			writer.close();
			
		}
		catch(Exception exc){
			exc.printStackTrace();
			System.exit(1);
		}
	}
	
	@SuppressWarnings(value = "unchecked")
	public List<Combination> sorting_and_selecting(List<Combination> combs){
		boolean use_set = false;
		List<Combination> res = new ArrayList<Combination>();
		
		if(!use_set){
			Collections.sort(combs);
			for(int i=0;i<MAX_NUMBER&&i<combs.size();i++){
				res.add(combs.get(combs.size()-i-1));
			}
			/*Iterator<Combination> it = combs.iterator();
			int counter = 0;
			while(counter<MAX_NUMBER && it.hasNext()){
				res.add(it.next());
				counter++;
			}*/
		}
		else{
		//	System.out.println("-> "+combs.size());
			Set<Combination> set = new TreeSet<Combination>(combs);
		//	System.out.println("-> "+set.size());
		
			Iterator<Combination> it = set.iterator();
			int counter = 0;
			while(it.hasNext()&&counter<MAX_NUMBER){
				res.add(it.next());
				counter++;
			}
		}
		return res;
			
	}
	
	public List<Combination> filter_splice_locations(List<Combination> combs){
		List<Combination> res = new ArrayList<Combination>();
		for(int i=0;i<combs.size();i++){
			Combination comb = combs.get(i);
			boolean test = true;
			String[] split1 = comb.description_1.split(" ");
			String[] split2 = comb.description_2.split(" ");
			int pos1 = Integer.parseInt(split1[2]);
			int pos2 = Integer.parseInt(split2[2]);
			if(pos1==0 || pos1==1)
				test = false;
			if(pos2==0 || pos2==1)
				test = false;
			if(test)
				res.add(comb);
			
		}
		
		return res;
	}
	
	public List<Combination> filter_dubbels(List<Combination> combs){
		List<Combination> res = new ArrayList<Combination>();
		for(int i=0;i<combs.size();i++){
			boolean test = true;
			Combination comb1 = combs.get(i);
			for(int j=i+1;j<combs.size();j++){
				Combination comb2 = combs.get(j);
				if(comb1.equals(comb2)){
					test = false;
					break;
				}
			}
			if(test)
				res.add(comb1);
		}
		return res;
	}
	
	public List<Combination> filter_overlapping(List<Combination> combs){
		List<Combination> res = new ArrayList<Combination>();
		for(int i=0;i<combs.size();i++){
			Combination comb1 = combs.get(i);
			if(!comb1.overlaps())
				res.add(comb1);
		}
		return res;
	}
	
	public List<Combination> removeDoubleCombinations(List<Combination> combs){
		List<Combination> res = new ArrayList<Combination>();
		Set<Combination> set = new HashSet<Combination>();
		
		for(int i=0;i<combs.size();i++){
			set.add(combs.get(i));
		}
		
		Iterator<Combination> it = set.iterator();
		while(it.hasNext())
			res.add(it.next());
		return res;
	}
	
	public void showResultingCombinations(List<Combination> combinations){
		new FunSiPFeatureGui(combinations);
	}
	
	public List<Combination> findDescriptions(List<Combination> combs){
		List<Combination> res = new ArrayList<Combination>();
		for(int i=0;i<combs.size();i++){
			Combination comb = combs.get(i);
			ClassificationFeature first_feature = features.get(comb.first_feature);
			ClassificationFeature second_feature = features.get(comb.second_feature);
			String firstDescription = first_feature.getLocationDescription(comb.first_position).toString();
			String secondDescription = second_feature.getLocationDescription(comb.second_position).toString();
			comb.description_1 = firstDescription;
			comb.description_2 = secondDescription;
			res.add(comb);
		}
		return res;
	}
	
	public void printResultingCombinations(List<Combination> combinations){
		for(int i=0;i<combinations.size();i++){
			Combination comb = combinations.get(i);
			ClassificationFeature first_feature = features.get(comb.first_feature);
			ClassificationFeature second_feature = features.get(comb.second_feature);
			
			String firstDescription = first_feature.getLocationDescription(comb.first_position).toString();
			String secondDescription = second_feature.getLocationDescription(comb.second_position).toString();
			
			System.out.println();
			System.out.println(firstDescription);
			System.out.println(secondDescription);
			System.out.println(comb.support);
			//System.out.println();
			
			comb.description_1 = firstDescription;
			comb.description_2 = secondDescription;
		}
		System.out.println("TOTAL : "+combinations.size());
	}
	
	public List<Combination> doCombinations(){
		List<Combination> result = new ArrayList<Combination>();
		
		//outer loops over all feature results
		for(int i=0;i<features.size();i++){
			int maxLength1 = extractedFeatures.get(0).get(i).size();
			for(int j=0;j<maxLength1;j++){
				System.out.print(".");
				//inner loops over all feature results
				for(int k=0;k<features.size();k++){
					int maxLength2 = extractedFeatures.get(0).get(k).size();
					for(int l=0;l<maxLength2;l++){
						
						if(i==k && j==l){
							//do nothing, is same feature
						}
						else{
							double support = 0;
							//across all positive data examples, get the datapoints.
							for(int m=0;m<extractedFeatures.size();m++){
								double val1 = extractedFeatures.get(m).get(i).get(j);
								double val2 = extractedFeatures.get(m).get(k).get(l);
								
								/*if(val1>0 && val2>0)
									support++;*/
								if(val1==0 && val2==0)
									support++;
							}
							support = support /extractedFeatures.size();
							
							if(support>MINIMUM_SUPPORT_FIRST){
								Combination comb = new Combination();
								comb.first_feature = i;
								comb.first_position = j;
								comb.second_feature = k;
								comb.second_position = l;
								comb.support = support; 
								result.add(comb);
							}
						}
						
					}
				}
				
			}
		}
		
		System.out.println();

		
		return result;
	}
	
	private void extractFeatures(List<String>posData,int splicesite){
		extractedFeatures.clear();
		try{
			List<String[]> all_types = getTypes(features);
			
			for(int i=0;i<posData.size();i++){
				String sequence = posData.get(i);
				String[] seqs = {sequence};
				List<List<Double>> sequenceFeatures = new ArrayList<List<Double>>();
				for(int j=0;j<features.size();j++){
					ClassificationFeature feat = features.get(j);
					String[] types = all_types.get(j);
					List<Double> feats = feat.extractFeatures(seqs, splicesite, types, null, null);
					sequenceFeatures.add(feats);
				}
				extractedFeatures.add(sequenceFeatures);
			}
		}
		catch(Exception exc){
			exc.printStackTrace();
			System.exit(1);
		}
	}
	
	private List<String[]> getTypes(List<ClassificationFeature> features){
		List<String[]> res = new ArrayList<String[]>();
		char[] DNA_alphabet = {'A','C','G','T'};
		for(int i=0;i<features.size();i++){
			ClassificationFeature feature = features.get(i);
			String [] types = null;
			if(feature.needCompositionalTypes()){
				types = ClassificationFeatureUtils.generateGeneralCompositionalTypes(feature.getLength(), DNA_alphabet);
			}
			res.add(types);
		}
		return res;
		
	}
	
	//features defined.
	private List<ClassificationFeature> defineFeatures(){
		
		ClassificationFeature feat_p_1 = new PositionalClassificationFeature();
		ClassificationFeature feat_p_2 = new PositionalClassificationFeature();
	//	ClassificationFeature feat_p_3 = new PositionalClassificationFeature();
	/*	ClassificationFeature feat_c_3 = new CompositionalClassificationFeature();
		ClassificationFeature feat_c_4 = new CompositionalClassificationFeature();
		ClassificationFeature feat_c_5 = new CompositionalClassificationFeature();
		ClassificationFeature feat_c_6 = new CompositionalClassificationFeature();*/
		
		try{
			feat_p_1.setParameters(new String("P 1 10 10").split(" "));
			feat_p_2.setParameters(new String("P 2 10 10").split(" "));
		//	feat_p_3.setParameters(new String("P 3 20 20").split(" "));
	/*		feat_c_3.setParameters(new String("C 3 20 20").split(" "));
			feat_c_4.setParameters(new String("C 4 20 20").split(" "));
			feat_c_5.setParameters(new String("C 5 20 20").split(" "));
			feat_c_6.setParameters(new String("C 6 20 20").split(" "));*/
		}
		catch(Exception exc){
			exc.printStackTrace();
			return null;
		}
		
		List<ClassificationFeature> feat2 = new ArrayList<ClassificationFeature>();
		feat2.add(feat_p_1);
		feat2.add(feat_p_2);
	//	feat2.add(feat_p_3);
	/*	feat2.add(feat_c_3);
		feat2.add(feat_c_4);
		feat2.add(feat_c_5);
		feat2.add(feat_c_6);*/
		
		return feat2;
	}
	
	
	public class Combination implements Comparable{
		public int first_feature 		= -1;
		public int second_feature		= -1;
		
		public int first_position 		= -1;
		public int second_position		= -1;
		
		public double support			= -1.0;
		
		public String description_1 			= null;
		public String description_2			= null;
		
		public Combination(){
			super();
		}
		
		public String toString(){
			return description_1+"/"+description_2;
		}
		
		public int compareTo(Object obj){
			Combination comb = (Combination)obj;
			if(this.support < comb.support)
				return -1;
			else if(this.support == comb.support)
				return 0;
			else 
				return 1;
		}
		
		public boolean overlaps(){
			boolean res = false;
			if(description_1==null || description_2==null){
				return false;
			}
			
			String[] this_split1 = this.description_1.split(" ");
			String[] this_split2 = this.description_2.split(" ");
			
			if(this_split1[3].equals(this_split2[3])){
				int s1 = Integer.parseInt(this_split1[2]);
				int s2 = Integer.parseInt(this_split2[2]);
				if(s1==s2||(s1+1)==s2||s1==(s2+1))
					res = true;
			}
			
			return res;
		}
		
		@Override
		public boolean equals(Object ob){
			if(!(ob instanceof Combination))
				return false;
			
			Combination comb = (Combination)ob;
			boolean res = false;
			if(this.first_feature==comb.first_feature && 
					this.second_feature==comb.second_feature&&
					this.first_position==comb.first_position&&
					this.second_position==comb.second_position){
				res = true;
			}
			else if(this.first_feature==comb.second_feature&&
					this.second_feature==comb.first_feature&&
					this.first_position==comb.second_position&&
					this.second_position==comb.first_position){
				res = true;
			}
			else{
				res = false;
			}
			return res;
		}
	}
}
