/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

package main;



/**
 * This class is just meant as the main-contact point when the
 * jar-file is launched. This class then performs the necessary
 * commandline parameter checking, and launches the selected 
 * targets from the jar file.
 * @author Michiel Van Bel
 *
 */
public class FunSiPJar {
	
	/**
	 * Accesible string indicating the prefix used for selecting outputdirectories.
	 */
	public static String USER_DIR = "";
	
	/**
	 * Accesible boolean indicating whether or not the program is running as a jar-file.
	 */
	public static boolean RUN_FROM_JAR = false;
	
	
	/**
	 * Main method (indicated by the manifest file). Launches the content
	 * of the jar-file. In this case, launches the FunSiP-program:
	 * either the gui version of the direct non-interactive version (specified
	 * by the -c flag).
	 * @param args The commandline parameters, used when launching the 
	 * jar-file.
	 */
	public static void main(String []args){
		//try to find the user directory, then place a properties file in there.
		
		/*
		 * TODO:  WARNING: this is come code that changes the normal 
		 * User directory to that of the user home directory. 
		 * Removed because it is some stupid retarded %&##$ 
		 */
		
		
		/*
		Properties prop = System.getProperties();
		String homeDir = prop.getProperty("user.home");
		if(homeDir!=null){
			if(!homeDir.endsWith("\\")&&!homeDir.endsWith("/"))
				homeDir+="/";
			USER_DIR = homeDir+"FunSiP/";
			File file = new File(USER_DIR);
			if(!file.exists()){
				file.mkdirs();
				//System.out.println("creation dirs succeeded? "+c);
			}
			else{
				//System.out.println("No dir creation neccesary");
			}
		}
		else{
			System.out.println("Error while obtaining directory that is suitable for temporary" +
					"data storage.\n");
			System.exit(2);
		}
		*/
		
		new FunSiPJar(args);
	}
	
	/**
	 * Constructor method for FunSiPJar. Checks whether or not the
	 * -c flag is defined. If so, launch FunSiP with remaining parameters. 
	 * If not, launch FunSiPGui.
	 */
	public FunSiPJar(String []args){
		/*
		 * Preload all the jar-files from the lib-dir into the classpath,
		 * because it cannot be done by setting the class-path variable in 
		 * the manifest file :-(
		 * We load every library in a seperate try-catch block, in order to
		 * reduce the error-overhead when loading a class fails.
		 */
		//loadLibrary("./libs/log4j-1.2.14.jar");
		//loadLibrary("./libs/weka.jar");
		RUN_FROM_JAR = true;
		if(args.length==0){
			FunSiPGui.main(args);
		}
		else{
			if(args[0].equals("-c")){ //commandline mode
				String[] newArgs = new String[args.length-1];
				for(int i=1;i<args.length;i++)
					newArgs[i-1]=args[i];
				FunSiP.main(newArgs);
			}
		}
		
	}
	
	/**
	 * Changes the pathname when running from a jar-file (some 
	 * indicators change for some weird reason).
	 * @param path The original pathname.
	 * @return The new pathname.
	 */
	public static String setCorrectFilePath(String path){
		if(RUN_FROM_JAR){
			if(path.startsWith("./")||path.startsWith(".\\")){
				String t = path.substring(1,path.length());
				return t;
			}
		}
		return path;
	}
}
