package test.functions;

import java.util.ArrayList;
import java.util.List;
import java.util.Random;

public class FunctionUtils {

	public static final String[] possible_operators = {"-","+","/","*"};
	
	//always have 1 more variable index then operator
	private List<Integer> variable_indices 	= new ArrayList<Integer>();
	private List<String> current_operators 	= new ArrayList<String>();
	private int num_vars					= 0;
	
	private boolean DEBUG =  false;
	
	
	public static void main(String[] args){
		FunctionUtils fu = new FunctionUtils();
		Function f = fu.generateNextFunction(10,7);
		System.out.println(f.toString());
		int t = f.getNumberOfVariables();
		double [] temp = new double[t];
		for(int i=0;i<t;i++)
			temp[i] = i+1;
		double result = f.executeFunction(temp);
		System.out.println(result);
	}
	
	public FunctionUtils(){
	}
	
	public Function generateNextFunction(int num_vars,int mul_vars){
		Function f = null;
		Random random = new Random();
		
		//define number of variables to be used by the function
		int number_variables = random.nextInt(num_vars)+2;
		this.num_vars = number_variables;
		if(DEBUG)System.out.println("NUMBER OF DIFFERENT VARIABLES : "+this.num_vars);
		
		//define how many times each variable can be used in the function
		int[] variable_times = new int[number_variables];
		for(int i=0;i<number_variables;i++)
			variable_times[i] = random.nextInt(mul_vars)+1;
		int total_num_variables = 0;
		for(int i=0;i<number_variables;i++)
			total_num_variables += variable_times[i];
		if(DEBUG)System.out.println("TOTAL NUMBER OF VARIABLES : "+total_num_variables);
		
		//define the number of operators to be used
		int num_operators = total_num_variables-1;
		String[] operators = new String[num_operators];
		for(int i=0;i<num_operators;i++){
			operators[i] = possible_operators[random.nextInt(4)];
		}
		List<String> ops = new ArrayList<String>();
		for(int i=0;i<operators.length;i++){
			ops.add(operators[i]);
		}
		if(DEBUG)System.out.println("OPERATORS : "+ops);
		
		//create the first draft of the function
		List<Integer> functionVariableIndices = new ArrayList<Integer>();
		
		int count = 0;
		while(count<total_num_variables){
			int index = random.nextInt(number_variables);
			if(variable_times[index]!=0){
				functionVariableIndices.add(index);
				variable_times[index]--;
				count++;
			}
		}
		
		this.variable_indices = functionVariableIndices;
		this.current_operators = ops;
		
		f = new AbstractFunction(this);
		return f;
	}
	
	
	public String currentFunction(){
		StringBuffer buf = new StringBuffer();
		for(int i=0;i<current_operators.size();i++){
			buf.append(getArrayString(variable_indices.get(i))+" "+current_operators.get(i)+" ");
		}
		buf.append(getArrayString(variable_indices.get(variable_indices.size()-1)));
		return buf.toString();
	}
	private String getArrayString(int index){
		return "a["+index+"]";
	}
	
	public double executeCurrentFunction(double[] a){
		return executeCurrentFunction(a,variable_indices,current_operators);
	}
	
	public double executeCurrentFunction(double[] a,List<Integer> indices, List<String> operators){
		double result = 0;
		//first, fill in the list of indices with actual data
		List<Double> values = new ArrayList<Double>();
		for(int i=0;i<indices.size();i++){
			values.add(a[indices.get(i)]);
		}
		while(operators.contains("*")||operators.contains("/")){
			doFirstOperations(values,operators);
		}
		while(operators.size()!=0){
			doSecondOperations(values,operators);
		}
		
		result = values.get(0);
		return result;
	}
	
	public void doSecondOperations(List<Double> values,List<String> operators){
		for(int i=0;i<operators.size();i++){
			String op = operators.get(i);
			double val1 = values.get(i);
			double val2 = values.get(i+1);
			double res = 0.0;
			if(op.equals("+"))
				res = val1+val2;
			else if(op.equals("-"))
				res = val1-val2;
			values.set(i, res);
			values.remove(i+1);
			operators.remove(i);
		}
	}
	
	public void doFirstOperations(List<Double>values,List<String>operators){
		for(int i=0;i<operators.size();i++){
			String op = operators.get(i);
			if(op.equals("*")){
				double val1 = values.get(i);
				double val2 = values.get(i+1);
				double res = val1*val2;
				values.set(i, res);
				values.remove(i+1);
				operators.remove(i);
				break;	
			}
			else if(op.equals("/")){
				double val1 = values.get(i);
				double val2 = values.get(i+1);
				double res = val1;
				if(val2!=0.0)
					res = val1/val2;
				values.set(i, res);
				values.remove(i+1);
				operators.remove(i);
				break;
			}
			else{
				//do nothing
			}
		}
	}
	
	
	public int getCurrentFunctionVariables(){
		return num_vars;
	}
	
}