/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
package util.classificationFeatures.implementations;

import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics2D;
import java.util.ArrayList;
import java.util.List;

import org.apache.batik.svggen.SVGGraphics2D;

import util.classificationFeatures.ClassificationFeatureDrawing;
import util.classificationFeatures.ClassificationFeatureUtils;
import util.classificationFeatures.IncorrectClassificationFeatureException;

public class CompositionalClassificationFeatureDrawing implements ClassificationFeatureDrawing{
	
	private CompositionalClassificationFeature ccf = null;
	
	
	public CompositionalClassificationFeatureDrawing(CompositionalClassificationFeature ccf){
		this.ccf = ccf;
	}
	
	/**
	 * This method draws the 
	 * @param g2d
	 * @param list
	 * @throws IncorrectClassificationFeatureException
	 */
	public void draw(Graphics2D g2d, List<Integer> list ) throws IncorrectClassificationFeatureException{
		if(list.size()!=ccf.getNumberOfAttributes()){
			throw new IncorrectClassificationFeatureException();
		}
	}
	
	/**
	 * This method draws the scores assigned to each feature,
	 * that is generated by this corresponding ClassificationFeature. 
	 * 
	 * The normal idea is to let certain FeatureSelection methods assign
	 * a score to each feature, and then draw these scores in a
	 * classificationfeature dependend way.
	 * 
	 * @param svg
	 * @param list
	 * @throws IncorrectClassificationFeatureException
	 */
	public void draw(SVGGraphics2D svg, List<Double> list, int blocksize, int splicesite_increase ) throws IncorrectClassificationFeatureException{
		if(ccf==null||list.size()!=ccf.getNumberOfAttributes()){
			throw new IncorrectClassificationFeatureException();
		}
		//get the highest score, needed for better drawing
		double maxvalue = 0;
		for(int i=0;i<list.size();i++){
			double d = list.get(i);
			if(d>maxvalue)
				maxvalue = d;
		}	
		
		List<List<Double>> up_down_scores = splitScoresUpDown(list);
		List<Double> upScores = up_down_scores.get(0);
		List<Double> downScores = up_down_scores.get(1);
		
		int length = ccf.getLength();
		String[] types = ClassificationFeatureUtils.generateCompositionalTypes(length);
		
		//Some initial drawing
		svg.setFont(new Font(null,Font.BOLD,20));
		svg.drawString(ccf.toString(),20, 20);
		svg.setFont(new Font(null,Font.PLAIN,blocksize/2));
		
		svg.setFont(new Font(null,Font.ITALIC,15));
		svg.drawString("Upstream", 10, 40);
		
		int length2 = (int)(Math.pow(4,length))/4;
		int column = 0;
		for(int i=0;i<upScores.size();i+=length2){
			for(int j=0;j<length2;j++){
				double score = upScores.get(i+j);
				Color c = createColor(createColorValue(score,maxvalue));
				
				svg.setFont(new Font(null,Font.PLAIN,blocksize/2));
				svg.drawString(types[i+j], 5+(blocksize+120)*column,65+blocksize*j);
				svg.setColor(c);
				svg.fillRect(100+(blocksize+120)*column,50+blocksize*j,blocksize,blocksize);
				svg.setColor(Color.black);
				svg.drawRect(100+(blocksize+120)*column,50+blocksize*j,blocksize,blocksize);
				
				//drawing of k
				svg.setColor(Color.black);
				svg.fillRect(110+blocksize+(blocksize+120)*column, 45+blocksize*j, 10, blocksize+10);
			}
			column++;
		}
		
		svg.setFont(new Font(null,Font.ITALIC,15));
		svg.drawString("Downstream",10,blocksize*length2+80);
		column = 0;
		
		for(int i=0;i<downScores.size();i+=length2){
			for(int j=0;j<length2;j++){
				double score = downScores.get(i+j);
				Color c = createColor(createColorValue(score,maxvalue));
				svg.setFont(new Font(null,Font.PLAIN,5));
				svg.drawString(types[i+j], 5+(blocksize+120)*column,blocksize*length+115+blocksize*j);
				svg.setColor(c);
				svg.fillRect(100+(blocksize+120)*column,blocksize*length+100+blocksize*j,blocksize,blocksize);
				svg.setColor(Color.black);
				svg.drawRect(100+(blocksize+120)*column,blocksize*length+100+blocksize*j,blocksize,blocksize);
				
				//drawing of k
				svg.setColor(Color.black);
				svg.fillRect(110+blocksize+(blocksize+120)*column, blocksize*length+95+blocksize*j, 10, blocksize+10);
			}
			column++;
		}
		
		for(int i=0;i<250;i++){
			svg.setColor(createColor(i));
			svg.fillRect(40+i*2,140+2*length*blocksize,2,30);
		}
		svg.setColor(Color.black);
		svg.drawString("0",35,190+2*length*blocksize);
		svg.drawString("1",535,190+2*length*blocksize);
	}
	
	public Color createColor(double value){
		double val = 250*value;
		int red = (int)(val);
		int green = (int)(250-val);
		double b = (-2.0/125)*val*val + 4*val;
		int blue = (int)b;
		return new Color(red,green,blue);
	}
	
	
	public Color createColor(int value){
		double temp = ((double)value/250);
		return createColor(temp);
	}
	
	public double createColorValue(double value, double maxvalue){
		value = value/maxvalue;
		/*if(value!=0.0){
		value+=0.01;
		value = Math.pow(2,value)/2;
		if(value>1)
			value = 1.0;
		}*/
		return value;
	}

	
	/*
	 * This method splits the list of scores in 2 (upstream, downstream) , 
	 * just in 2 (upstrreamsize == downstreamsize)
	 */
	private List<List<Double>> splitScoresUpDown(List<Double> scores){
		List<List<Double>> result = new ArrayList<List<Double>>();
		
		List<Double> up = new ArrayList<Double>();
		List<Double> down = new ArrayList<Double>();
		
		for(int i=0;i<scores.size()/2;i++)
			up.add(scores.get(i));
		for(int i=scores.size()/2;i<scores.size();i++)
			down.add(scores.get(i));
		
		result.add(up);
		result.add(down);
		
		return result;
		
	}

}
