/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
package util.conversions.implementations;

import util.conversions.Conversion;

public class Test_2_Conversion implements Conversion{
	
	private boolean clockwise = true;
	
	public Test_2_Conversion(){
		
	}
	
	/**
	 * Returns the name of conversion. This name should be unique
	 * among all other conversionnames, active or not (just in case).
	 * @return The unique name of the conversion
	 */
	public String getName(){
		return "TEST_2_CONVERSION";
	}
	
	
	/**
	 * This method performs the actual conversion from one string
	 * to another. This can for example be the translation from RNA
	 * to pyrimidine/purine sequence, or the translation from the
	 * primary structure to the secondary structure.
	 * @param s The original sequence
	 * @return The translated string
	 */
	public String makeConversion(String s){
		int splicesite = s.length()/2;
		StringBuffer result = new StringBuffer();
		
		result.append(s.charAt(splicesite));
		
		int start = 1;
		int stop = s.length()/2;
		if(s.length()%2!=0)
			stop++;
		for(int i=start;i<stop;i++){
			if(clockwise){
				result.append(s.charAt(splicesite-i));
				result.append(s.charAt(splicesite+i));
			}
			else{
				result.append(s.charAt(splicesite+i));
				result.append(s.charAt(splicesite-i));
			}
		}
		
		if(s.length()%2==0)
			result.append(s.charAt(0));
			
		return result.toString();
	}
	
	
	/**
	 * This method sets the possible options for this conversion.
	 * These options are of course dependend on the type of conversion,
	 * so this method is as general as possible.
	 * @param args The possible options (name+value)
	 * @return  Whether setting the options succeeded (true) or not (false).
	 */
	public boolean setOptions(String[] args){
		if(args.length!=2){
			return false;
		}
		if(!args[0].toUpperCase().equals("TEST2"))
			return false;
			
		String t = args[1].toUpperCase();
		if(t.equals("CLOCK_WISE"))
			clockwise = true;
		else if(t.equals("COUNTER_CLOCK_WISE"))
			clockwise = false;
		else
			return false;
		return true;
	}
	
	/**
	 * Returns the parameter name of the conversion. This is usefull for
	 * compatibility with gui's and other programs which rely on the
	 * parameter for identification.
	 * @return String with parametername (unique).
	 */
	public String getParamName(){
		return "TEST2";
	}

	/**
	 * This method returns the number of different characters present in
	 * the final representation after a conversion on a sequence has 
	 * occured (excluding the 'N' character).
	 * @return The number of different characters
	 */
	public int getNumberOfDifferentCharacters(){
		return 4;
	}
	
	
	/**
	 * This method returns all possible different characters that occur
	 * after the conversion of a sequence. This is of course a description
	 * of the alphabet used by the conversion. The 'N' character is 
	 * excluded.
	 * @return All posible characters used by the conversion 
	 */
	public char[] getDifferentCharacters(){
		char[] res = {'A','C','G','T'};
		return res;
	}

	public static void main(String[] args){
		Conversion t = new Test_2_Conversion();
		String seq = "AACAGCTAGCTAGGATCGAGAGGACTTATAAGGCT";
		String res = t.makeConversion(seq);
		
		System.out.println(seq.length());
		System.out.println(seq);
		System.out.println(res);
	}
	
}
