/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
package util.io;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.util.ArrayList;
import java.util.List;

import org.apache.log4j.Logger;

import base.ClassificationAction;


import util.RocCurveData;
import util.results.CrossValidationResult;

public class CrossValidationOutput {

	private List<CrossValidationResult> results = new ArrayList<CrossValidationResult>();
	private List<Integer> posTrainList = new ArrayList<Integer>();
	private List<Integer> negTrainList = new ArrayList<Integer>();
	private List<Integer> posTestList  = new ArrayList<Integer>();
	private List<Integer> negTestList  = new ArrayList<Integer>();
	
	private List<String> extraInfo 		= null;
	private String extraInfoLabel 		= null;
	
	private ClassificationAction ca = null;
	private Logger logger = null;
	
	public CrossValidationOutput(ClassificationAction ca, Logger logger){
		this.ca = ca;
		this.logger = logger;
	}
	
	public void writeResults(File outputFile,List<RocCurveData> nameList){
		try{
			
			BufferedWriter writer = new BufferedWriter(new FileWriter(outputFile));
			if(extraInfoLabel!=null&&extraInfo!=null&&extraInfo.size()==results.size())
				writer.write(extraInfoLabel+"\t");
			writer.write("POS_TEST_EXAMP \t");
			writer.write("NEG_TEST_EXAMP \t");
			writer.write("POS_TRAIN_EXAMP\t");
			writer.write("NEG_TRAIN_EXAMP\t");
			writer.write("TOTAL_POSITIVES\t");
			writer.write("TRUE_POSITIVES \t");
			writer.write("FALSE_NEGATIVES\t");
			writer.write("TOTAL_NEGATIVES\t");
			writer.write("TRUE_NEGATIVES \t");
			writer.write("FALSE_POSITIVES\t");
			writer.write("ERROR_RATE\t");
			writer.write("ACCURACY\t");
			writer.write("SENSITIVITY\t");
			writer.write("FP_RATE\t");
			writer.write("SPECIFICITY\t");
			writer.write("CORR_COEFF\t");
			writer.write("APPROX_CORREL\t");
			writer.write("F1_MEASURE\t");
			writer.write("FP5%_MEASURE\t");
			writer.write("AUC\t");
			writer.write("\n");
			
			for(int i=0;i<results.size();i++){
				if(extraInfoLabel!=null&&extraInfo!=null&&extraInfo.size()==results.size())
					writer.write(extraInfo.get(i)+"\t");
				CrossValidationResult res = results.get(i);
				writer.write(posTestList.get(i)+"\t");
				writer.write(negTestList.get(i)+"\t");
				writer.write(posTrainList.get(i)+"\t");
				writer.write(negTrainList.get(i)+"\t");
				writer.write(res.getTotalPositives()+"\t");
				writer.write(res.getTruePositives()+"\t");
				writer.write(res.getFalseNegatives()+"\t");
				writer.write(res.getTotalNegatives()+"\t");
				writer.write(res.getTrueNegatives()+"\t");
				writer.write(res.getFalsePositives()+"\t");
				writer.write(res.getError_rate()+"\t");
				writer.write(res.getAccuracy2()+"\t");
				writer.write(res.getSensitivity()+"\t");
				writer.write(res.getFp_rate()+"\t");
				writer.write(res.getSpecificity()+"\t");
				writer.write(res.getCorrelation_coefficient()+"\t");
				writer.write(res.getApproximate_correlation()+"\t");
				writer.write(res.getF1_measure()+"\t");
				writer.write(res.getFp5_measure()+"\t");
				writer.write(res.getAuc()+"\t");
				writer.write("\n");
				
				String dir = ca.getOutputdirectory()+"roc/";
				File f = new File(dir);
				if(!f.exists())
					f.mkdir();
				String name = outputFile.getName()+"_"+negTrainList.get(i)+".roc";
				writeRocCurveData(res.getRoc_tp_rate(),res.getRoc_fp_rate(),dir+name);
				nameList.add(new RocCurveData(name,negTrainList.get(i)));
			}
			writer.flush();
			writer.close();
		}
		catch(Exception exc){
			logger.error(exc);
			exc.printStackTrace();
		}
	}
	public void writeRocCurveData(double[] tp_rate_data, double [] fp_rate_data,String fileName){
		try{
			BufferedWriter writer = new BufferedWriter(new FileWriter(new File(fileName)));
			writer.write("#fp_rate \t tp_rate\n");
			for(int i=0;i<tp_rate_data.length;i++){
				writer.write(fp_rate_data[i]+"\t"+tp_rate_data[i]+"\n");
			}
			writer.flush();
			writer.close();
		}
		catch(Exception exc){
			logger.error(exc);
			exc.printStackTrace();
		}
	}
	
	
	public void addResult(CrossValidationResult cvs,int posTrain,int negTrain,int posTest, int negTest){
		//cvs.computeDeductedData();
		results.add(cvs);
		posTrainList.add(posTrain);
		negTrainList.add(negTrain);
		posTestList.add(posTest);
		negTestList.add(negTest);
	}
	
	public void clear(){
		results.clear();
	}

	public List<String> getExtraInfo() {
		return extraInfo;
	}

	public void setExtraInfo(List<String> extraInfo) {
		this.extraInfo = extraInfo;
	}

	public String getExtraInfoLabel() {
		return extraInfoLabel;
	}

	public void setExtraInfoLabel(String extraInfoLabel) {
		this.extraInfoLabel = extraInfoLabel;
	}	
}
