/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
package util.parsers;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import util.Util;

public class ExtractTrainingData {

	public static String DIRECTORY = "./origdata/";
	public static final int SPLICESITE = 150;
	
	public enum DATATYPES{
		ECTOCARPUS,MICROMONAS,OSTREOCOCCUS
	}
	
	public static void main(String [] args){
		
		//new ExtractTrainingData();
		new ExtractTrainingData(args);
	}
	
	public ExtractTrainingData(boolean temp){
		//do shit
	}

	public ExtractTrainingData(){
		File f = new File("./fasta2/pierre_Ch2_60K.artemis");
		GeneTrainingData dat = extractFromFileMicromonas(f);
		System.out.println(dat.getFileName());
		String seq = dat.getSequence();
		System.out.println(seq.substring(0,10)+"...");
		System.out.println("..."+seq.substring(seq.length()-10,seq.length()));
		List<GeneData> list = dat.getGeneData();
		for(int i=0;i<list.size();i++){
			GeneData we = list.get(i);
			List<Integer> ss= we.getSplicesites();
			System.out.print(we.isForwardStrand()+"\t");
			for(int j : ss){
				System.out.print(j+"\t");
			}
			System.out.println();
		}
		System.out.println(seq);
	}
	
	public ExtractTrainingData(String[] args){
		System.out.println("Extracting data from training files");
		try{

			File file = new File(args[0]);
			if(!file.exists()){
				System.out.println("Dir does not exist");
				return;
			}
				
			
			if(!file.isDirectory()){
				System.out.println("file is not directory");
				return;
			}
			
			BufferedWriter pd_w = new BufferedWriter(new FileWriter(new File(DIRECTORY+"donors.pos")));
			BufferedWriter nd_w = new BufferedWriter(new FileWriter(new File(DIRECTORY+"donors.neg")));
			BufferedWriter pa_w	= new BufferedWriter(new FileWriter(new File(DIRECTORY+"acceptors.pos")));
			BufferedWriter na_w = new BufferedWriter(new FileWriter(new File(DIRECTORY+"acceptors.neg")));
				
			DATATYPES type = parseType(args[1]);
			
			File[] files = file.listFiles();
			for(int i=0;i<files.length;i++){
				File temp = files[i];
				GeneTrainingData data = extractFromFile(temp,type);
				GeneTestResult res = testData(data);
				if(res.isCorrectTest())
					extractFromData(data,pd_w,nd_w,pa_w,na_w);
				else
					res.printTestResult();
			}
			
			pd_w.flush();
			nd_w.flush();
			pa_w.flush();
			na_w.flush();
			
			pd_w.close();
			nd_w.close();
			pa_w.close();
			na_w.close();
		}
		catch(Exception exc){
			System.err.println(exc);
			exc.printStackTrace();
		}
		System.out.println("Data extraction completed.");
	}
	
	public DATATYPES parseType(String s){
		DATATYPES type = null;
		if(s.equals("ECTOCARPUS"))
			type = DATATYPES.ECTOCARPUS;
		else if(s.equals("MICROMONAS"))
			type = DATATYPES.MICROMONAS;
		else if(s.equals("OSTREOCOCCUS"))
			type = DATATYPES.OSTREOCOCCUS;
		return type;
	}
	
	public void extractFromData(GeneTrainingData data, BufferedWriter posDon,BufferedWriter negDon,
			BufferedWriter posAcc, BufferedWriter negAcc) throws IOException{
		System.out.println(data.getFileName());
		List<GeneData> geneDataList = data.getGeneData();
		String sequence = data.getSequence();
		List<Integer> fullAcc = new ArrayList<Integer>();
		List<Integer> fullDon = new ArrayList<Integer>();
		for(int i=0;i<geneDataList.size();i++){
			GeneData geneData = geneDataList.get(i);
			boolean forwardStrand = geneData.isForwardStrand();
			List<Integer> donors = geneData.getDonorSites();
			List<Integer> acceptors = geneData.getAcceptorSites();
			if(forwardStrand){
				for(int d : donors) fullDon.add(d);
				for(int a : acceptors) fullAcc.add(a);
			}
			//negAcc.write(data.getFileName()+"\n");
			extractPositiveAcceptors(acceptors,sequence,posAcc,forwardStrand);
			extractPositiveDonors(donors,sequence,posDon,forwardStrand);
		}
		extractNegativeAcceptors(fullAcc,sequence,negAcc);
		extractNegativeDonors(fullDon,sequence,negDon);
		
	}
	
	
	
	public void extractNegativeDonors(List<Integer> donors,String sequence,BufferedWriter negDon) throws IOException{
		for(int i=0;i<sequence.length()-1;i++){
			String temp = sequence.substring(i,i+2);
			temp = temp.toUpperCase();
			if(temp.equals("GT")/*||temp.equals("GC")*/){
				if(!donors.contains(i)){
					String le = extractUpDown(i,sequence,true);
					negDon.write(le+"\n");
				}
			}
		}
	}
	
	public void extractNegativeAcceptors(List<Integer>acceptors,String sequence,BufferedWriter negAcc) throws IOException{
		for(int i=0;i<sequence.length()-1;i++){
			String temp = sequence.substring(i,i+2);
			temp = temp.toUpperCase();
			if(temp.equals("AG")){
				if(!acceptors.contains(i+3)){
					String le = extractUpDown(i+2,sequence,true);
					negAcc.write(le+"\n");
				}
			}
		}
	}
	
	
	public void extractPositiveDonors(List<Integer> donors,String sequence,BufferedWriter posDon, boolean forward) throws IOException{
		int got = 0;
		if(forward){ //get all forward data
			for(int d:donors){
				String temp = sequence.substring(d,d+2);
				temp = temp.toUpperCase();
				if(temp.equals("GT") /*||temp.equals("GC")*/){
					String le = extractUpDown(d,sequence,forward);
					posDon.write(le+"\n");
					got++;
				}
			}
		}
		else{ //get only the positive splice sites on reverse strand
			for(int d: donors){
				String temp = sequence.substring(d-3,d-1);
				temp = temp.toUpperCase();
				if(temp.equals("AC")/*||temp.equals("GC") */){
					String le = extractUpDown(d-1,sequence,forward);
					posDon.write(le+"\n");
					got++;
				}
			}
		}
	/*	System.out.println("Donors : ");
		for(int d : donors){
			System.out.print(d +"\t");
		}
		System.out.println();
		System.out.println("Found :"+got+"\t"+donors.size());*/
	}
	
	
	public void extractPositiveAcceptors(List<Integer> acceptors,String sequence,BufferedWriter posAcc, boolean forward) throws IOException{
		int got = 0;
		if(forward){
			
			for(int a : acceptors){
				String temp = sequence.substring(a-3,a-1);
				temp = temp.toUpperCase();
				if(temp.equals("AG")){
					String le = extractUpDown(a-1,sequence,forward);
					posAcc.write(le+"\n");
					got++;
				}
			}
		}
		else{ //get only the positive splice sites on reverse strand
			for(int a : acceptors){
				String temp = sequence.substring(a,a+2);
				temp = temp.toUpperCase();
				if(temp.equals("CT")){
					String le = extractUpDown(a,sequence,forward);
					posAcc.write(le+"\n");
					got++;
				}
			}
		}
	/*	System.out.println("Acceptors :");
		for(int a : acceptors)
			System.out.print(a+"\t");
		System.out.println();
		System.out.println("Acceptors : "+got+"\t"+acceptors.size());*/
	}
	
	
	
	public String extractUpDown(int place,String sequence,boolean forward){
		int start = 0;
		int stop = 0;
		String res = "";
		boolean addLast = false;
		if(SPLICESITE>place){
			start = 0;
			for(int i=0;i<SPLICESITE-place-1;i++)
				res+="N";
		}
		else{
			start = place-SPLICESITE+1;
		}
		
		if(place+SPLICESITE>=sequence.length()){
			stop = sequence.length();
			addLast = true;
		}
		else{
			stop = place+SPLICESITE-1;
		}
		
		res+=sequence.substring(start,stop);
		if(addLast){
		//	System.err.println("too short "+res);
			
			for(int i=sequence.length();i<place+SPLICESITE-1;i++)
				res+="N";
		}
		
		if(!forward)
			res = Util.toForwardStrand(res);
		
		res = res.toUpperCase();
		
		return res;
	}
	
	
	
	public GeneTestResult testData(GeneTrainingData data){
		GeneTestResult res = new GeneTestResult();
		List<GeneData> list = data.getGeneData();
		String sequence = data.getSequence();
		for(int i=0;i<list.size();i++){
			GeneData geneData = list.get(i);
			List<Integer> ss = geneData.getSplicesites();
			List<Integer> donorSS = new ArrayList<Integer>();
			List<Integer> acceptorSS = new ArrayList<Integer>();
			for(int j=1;j<ss.size()-1;j+=2){
				if(geneData.isForwardStrand()){
					donorSS.add(ss.get(j));
					acceptorSS.add(ss.get(j+1));
				}
				else{
					acceptorSS.add(ss.get(j));
					donorSS.add(ss.get(j+1));
				}
			}
			geneData.setAcceptorSites(acceptorSS);
			geneData.setDonorSites(donorSS);
			
			List<String>bounds = new ArrayList<String>();
			for(int j=1;j<ss.size()-1;j++){
				String dinuc = "";
				int splice = ss.get(j);
			//	System.out.print(splice+" ");
				if(j%2==0){
					dinuc = sequence.substring(splice-3,splice-1);
				}
				else{
					dinuc = sequence.substring(splice,splice+2);
				}
				bounds.add(dinuc);
			//	System.out.println(dinuc);
			}
			geneData.setBoundaries(bounds);
			//System.out.println()
		}
		
		res.setCorrectTest(true);
		return res;
	}
	
	
	
	
	public GeneTrainingData extractFromFile(File f, DATATYPES type){
		GeneTrainingData data = null;
		try{
			switch(type){
			case MICROMONAS :data = extractFromFileMicromonas(f); break;
			case OSTREOCOCCUS : data = extractFromFileOstreococcus(f); break; 
			case ECTOCARPUS : data = extractFromFileEctocarpus(f); break;
			default : break;
			}
		}
		catch(Exception exc){
			System.err.println("Problems when extracting data from file "+f.getName());
			exc.printStackTrace();
		}
		return data;
	}
	
	
	public GeneTrainingData extractFromFileMicromonas(File f){
		GeneTrainingData data = new GeneTrainingData();
		try{
			data.setFileName(f.getName());
			BufferedReader reader = new BufferedReader(new FileReader(f));
			String s = reader.readLine();
			List<GeneData> geneDataList = new ArrayList<GeneData>();
			String sequence = "";
			
			while(s!=null&&!s.startsWith("SQ")){
				if(s.contains("CDS")){
					GeneData gd = new GeneData();
					HeaderClass hc = parseHeader2(s);
					gd.setSplicesites(hc.splicesites);
					gd.setForwardStrand(hc.forwardStrand);
					geneDataList.add(gd);
				}
				s = reader.readLine();
			}
			
			while(s!=null){
				if(s.startsWith("SQ")){
					int bp = parseSQ(s);
					data.setDefinedBP(bp);
				}
				else{
					String temp = filterEMBLSeq(s);
					sequence+=temp;
				}
				s = reader.readLine();
			}
			
			data.setGeneData(geneDataList);
			data.setSequence(sequence);
		}
		catch(Exception exc){
			System.err.println("Error while parsing file "+f.getName());
			exc.printStackTrace();
		}
		
		return data;
	}
	
	public GeneTrainingData extractFromFileOstreococcus(File f){
	//	System.out.println("Parsing file "+f.getName());
		GeneTrainingData data = new GeneTrainingData();
		try{
			data.setFileName(f.getName());
			List<GeneData> geneDataList = new ArrayList<GeneData>();
			String sequence = "";
			BufferedReader reader = new BufferedReader(new FileReader(f));
			String s = reader.readLine();
			
			while(s!=null && s.startsWith(">")){
				//if(s.contains("FT   CDS")){
					GeneData gd = new GeneData();
					List<String> numList = new ArrayList<String>();
					numList.add(s);
					
					if(!(s.indexOf("complement")==-1 && s.indexOf("join")==-1) && !s.endsWith(")")){
						s = reader.readLine();
						numList.add(s);
					}
					HeaderClass hc = null;
					if(numList.size()==1){
					//	System.out.println(s);
						hc = parseHeader2(s);
					//	System.out.println("Headerparser 2");
					}
					else{
						hc = parseHeader3(numList);
					//	System.out.println("Headerparser 3");
					}
					gd.setSplicesites(hc.splicesites);
					gd.setForwardStrand(hc.forwardStrand);
					geneDataList.add(gd);
				
				s = reader.readLine();
			}
		//	}
			System.out.println("Header parsed");
			System.out.println("Number of genes : "+geneDataList.size());
			
			
			//s = ">chromosome ... "
			s= reader.readLine();
		//	int counter = 0;
			StringBuffer buffer = new StringBuffer();
			while(s!=null){
				s = s.trim();
				//sequence+=s;
				buffer.append(s);
				s = reader.readLine();
		//		counter++;
		//		if(counter%1000==0)System.out.println("... +"+counter);
			}
			sequence = new String(buffer);
			data.setGeneData(geneDataList);
			data.setSequence(sequence);
			data.setDefinedBP(-1);
		//	System.out.println("Parsing file "+f.getName()+" done");
		//	System.out.println();
		}
		catch(Exception exc){
			System.err.println("Error while parsing file "+f.getName());
			exc.printStackTrace();
		}
		return data;
	}
	
	public GeneTrainingData extractFromFileEctocarpus(File f){
		GeneTrainingData data = new GeneTrainingData();
		try{
			BufferedReader reader = new BufferedReader(new FileReader(f));
			String s = reader.readLine();
			HeaderClass header = parseHeader1(s);
			s = reader.readLine(); //empty >
			s = reader.readLine();
			String sequence = "";
			
			while(s!=null){
				s = s.trim();
				sequence+=s;
				s = reader.readLine();
			}
			
			GeneData geneData = new GeneData();
			
			geneData.setForwardStrand(header.forwardStrand);
			geneData.setSplicesites(header.splicesites);
			List<GeneData> list = new ArrayList<GeneData>();
			list.add(geneData);
			
			data.setFileName(f.getName());
			data.setDefinedBP(-1);
			data.setSequence(sequence);
			data.setGeneData(list);
		}
		catch(Exception exc){
			System.err.println("Error while extracting data from file :"+f.getName());
			exc.printStackTrace();
		}
		
		return data;
	}
	
	public int parseSQ (String s ){
		int bp =0;
		StringBuffer sb = new StringBuffer();
		for(int i=0;i<s.indexOf("BP");i++){
			Character c = s.charAt(i);
			if(Character.isDigit(c))
				sb.append(c);
		}
		String p = new String(sb);
		bp = Integer.parseInt(p);
		return bp;
	}
	
	public String filterEMBLSeq(String s){
		StringBuffer buffer = new StringBuffer();
		String result = null;
		for(int i=0;i<s.length();i++){
			char c = s.charAt(i);
			if(c=='a'||c=='A'||c=='t'||c=='T'
				||c=='c'||c=='C'||c=='g'||c=='G')
				buffer.append(c);
		}
			
		result = new String(buffer);
		result = result.toUpperCase();
		return result;
	}
	
	
	public HeaderClass parseHeader2(String s){
		HeaderClass header = new HeaderClass();
		List<Integer> res= new ArrayList<Integer>();
		boolean reverse = false;
		
		if(s.indexOf("complement")==-1)
			reverse = false;
		else
			reverse = true;
		
		int firstIndex = s.indexOf("join")+5;
		if(s.indexOf("join")==-1&&s.indexOf("complement")!=-1)
			firstIndex = s.indexOf("(")+1;
		if(s.indexOf("join")==-1&&s.indexOf("complement")==-1){
			firstIndex = s.length();
			String[] split = s.split(" ");
			String fin = split[split.length-1];
			int temp = tryIntegerParse(fin.substring(0,fin.indexOf(".")));
			res.add(temp);
			temp = tryIntegerParse(fin.substring(fin.lastIndexOf(".")+1,fin.length()));
			res.add(temp);
		}
		
		int temp = 0;
		String t = "";
		for(int i=firstIndex;i<s.length();i++){
			char c = s.charAt(i);
			
			if(c==')'){
				temp = tryIntegerParse(t);
				res.add(temp);
				break;
			}
			else if(c==','){
				temp = tryIntegerParse(t);
				res.add(temp);
				t = "";
			}
			else if(c=='.'){
				temp = tryIntegerParse(t);
				res.add(temp);
				t = "";
				i++;
			}
			else 
				t+=c;
		}
		
		header.forwardStrand = !reverse;
		header.splicesites = res;
		return header;
	}
	
	public HeaderClass parseHeader3(List<String> list){
		HeaderClass header = new HeaderClass();
		List<Integer> res= new ArrayList<Integer>();
		boolean reverse = false;
		
		String s = list.get(0);
		
		if(s.indexOf("complement")==-1)
			reverse = false;
		else
			reverse = true;
		
		int firstIndex = s.indexOf("join")+5;
		if(s.indexOf("join")==-1&&s.indexOf("complement")!=-1)
			firstIndex = s.indexOf("(")+1;
		if(s.indexOf("join")==-1&&s.indexOf("complement")==-1){
			firstIndex = s.length();
			String[] split = s.split(" ");
			String fin = split[split.length-1];
			int temp = tryIntegerParse(fin.substring(0,fin.indexOf(".")));
			res.add(temp);
			temp = tryIntegerParse(fin.substring(fin.lastIndexOf(".")+1,fin.length()));
			res.add(temp);
		}
		
		int temp = 0;
		String t = "";
		for(int i=firstIndex;i<s.length();i++){
			char c = s.charAt(i);
			
			if(c==')'){
				temp = tryIntegerParse(t);
				res.add(temp);
				break;
			}
			else if(c==','){
				temp = tryIntegerParse(t);
				res.add(temp);
				t = "";
			}
			else if(c=='.'){
				temp = tryIntegerParse(t);
				res.add(temp);
				t = "";
				i++;
			}
			else 
				t+=c;
		}
		String s2 = list.get(1);
		t = "";
		String[] splitDu = s2.split(" ");
		s2 = splitDu[splitDu.length-1];
		for(int i=0;i<s2.length();i++){
			char c = s2.charAt(i);
			if(c==')'){
				temp = tryIntegerParse(t);
				res.add(temp);
				break;
			}
			else if(c=='.'){
				temp = tryIntegerParse(t);
				res.add(temp);
				t = "";
				i++;
			}
			else if(c==','){
				temp = tryIntegerParse(t);
				res.add(temp);
				t = "";
			}
			else
				t+=c;
		}
		
		header.forwardStrand = !reverse;
		header.splicesites = res;
		return header;
	}
	
	private static int tryIntegerParse(String s){
		int start = 0;
		int stop = s.length();
		if(s.startsWith(">"))
			start = 1;
		if(s.startsWith("<"))
			start = 1;
		
		return Integer.parseInt(s.substring(start,stop));
			
	}
	
	public HeaderClass parseHeader1(String s){
		HeaderClass header = new HeaderClass();
		
		List<Integer> res = new ArrayList<Integer>();
		boolean reverse = false;
		
		if(s.indexOf("complement")!=-1)
			reverse = true;
		else
			reverse = false;
		
		int firstIndex = s.indexOf("join")+5;
		if(s.indexOf("join")==-1)
			firstIndex = s.indexOf("(")+1;
		int temp = 0;
		String t = "";
		for(int i=firstIndex;i<s.length();i++){
			char c = s.charAt(i);
			
			if(c==')'){
				temp = Integer.parseInt(t);
				res.add(temp);
				break;
			}
			else if(c==','){
				temp = Integer.parseInt(t);
				res.add(temp);
				t = "";
			}
			else if(c=='.'){
				temp = Integer.parseInt(t);
				res.add(temp);
				t = "";
				i++;
			}
			else
				t+=c;
		}
		
	/*	System.out.println("Number of splicesites : "+res.size());
		for(int i=0;i<res.size();i++){
			System.out.print(res.get(i)+"\t");
		}
		System.out.println();*/
		header.forwardStrand = !reverse;
		header.splicesites = res;
		return header;
	}
	
	
	private class HeaderClass{
		public boolean forwardStrand = true;
		public List<Integer> splicesites = new ArrayList<Integer>();
	}
	
}
