/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
package util.wekaUtilities;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;

import org.apache.log4j.ConsoleAppender;
import org.apache.log4j.Level;
import org.apache.log4j.Logger;
import org.apache.log4j.SimpleLayout;

import weka.core.Instances;

public class Conversions {
	
	private Logger logger = null;
	
	public Conversions(Logger logger){
		this.logger = logger;
	}

	public void ntrToArffConversion(File ntr,File arff){
		try{
			BufferedReader reader = new BufferedReader(new FileReader(ntr));
			BufferedWriter writer = new BufferedWriter(new FileWriter(arff));
			
			String s = reader.readLine();
			boolean headerMade = false;
			while(s!=null){
				if(!headerMade){
					String header = makeHeaderFromNtr(s);
					writer.write(header);
					headerMade = true;
				}
				String[] split = s.split(" ");
				StringBuffer buffer = new StringBuffer();
				buffer.append("{");
				buffer.append("0 "+split[0]+",");
				for(int i=1;i<split.length;i++){
					String pre = split[i].substring(0,split[i].indexOf(":"));
					String post = split[i].substring(split[i].indexOf(":")+1,split[i].length());
					buffer.append(pre+" "+post);
					if(i!=split.length-1)
						buffer.append(",");
					else
						buffer.append("}\n");
				}
				writer.write(new String(buffer));
				s = reader.readLine();
			}
			
			reader.close();
			writer.close();
		}
		catch(Exception exc){
			
		}
	}
	
	private String makeHeaderFromNtr(String s){
		String[] dat = s.split(" ");
		String last = dat[dat.length-1];
		int lastAttrib = Integer.parseInt(last.substring(0,last.indexOf(":")));
		StringBuffer result = new StringBuffer();
		result.append("@relation arff_data\n");
		result.append("@attribute class_attrib {1,-1}\n");
		for(int i=1;i<=lastAttrib;i++){
			result.append("@attribute attrib_"+i+" numeric\n");
		}
		result.append("@data\n");
		return new String(result);
	}
	
	public Instances ntrToInstancesConversion(File ntr){
		Instances instances = null;
		if(ntr==null)
			return null;
		String arffFileName = ntr.getPath().substring(0,ntr.getPath().lastIndexOf("."))+".arff";
		File arff = new File(arffFileName);
		ntrToArffConversion(ntr,arff);
		try{
			FileReader reader = new FileReader(arff);
			instances = new Instances(reader);
			instances.setClassIndex(0);
			reader.close();
			arff.delete();
		}
		catch(Exception exc){
			logger.error(exc);
		}
		return instances;	
	}
	
	
	public static void main(String [] args){
		Logger logger = Logger.getLogger("woot");
		logger.setLevel((Level)Level.DEBUG);
		ConsoleAppender appender = new ConsoleAppender(new SimpleLayout());
		logger.addAppender(appender);
		Conversions conv = new Conversions(logger);
		File file = new File(args[0]);
		conv.ntrToInstancesConversion(file);
	}
	
}
