package var.experimenter;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.util.ArrayList;
import java.util.List;

public class ConfigurationFileReader {
	
	/*
	 * WARNING : MUST BE THE SAME IN HERE AND IN THE CONFIGURATION FILE
	 */
	public final static int NO_ARGS_OPTION = 0;
	public final static int SINGLE_STRING_OPTION = 1;
	public final static int SINGLE_DOUBLE_OPTION = 2;
	public final static int SINGLE_INTEGER_OPTION = 3;
	public final static int DUAL_INTEGER_OPTION = 4;
	public final static int SINGLE_SELECT_OPTION = 5;
	public final static int DUAL_SELECT_OPTION = 6;

	
	public static List<RangeOption> readConfigurationFile(String fileName){
		List<String[]> temp = readFromFile(fileName);
		List<RangeOption> res = parseOptions(temp);
		return res;
	}
	
	public static List<String[]> readFromFile(String fileName){
		List<String[]> result = new ArrayList<String[]>();
		try{
			File file = new File(fileName);
			BufferedReader reader = new BufferedReader(new FileReader(file));
			String s = reader.readLine();
			while(s!=null){
				if(!s.startsWith("#") && !s.equals("") && !s.equals(" ")){
					s = s.trim();
					String [] temp = s.split("\t");
					if(temp!=null)
						result.add(temp);
				}
				s = reader.readLine();
			}		
		}
		catch(Exception exc){
			exc.printStackTrace();
			System.err.println("Configuration file not parsed. Aborting program.");
			System.exit(1);
		}
		return result;
	}
	
	
	public static RangeOption createOption (String[] data) throws Exception{
		//removal of the "-signs
		for(int i=0;i<data.length;i++){
			if(data[i].startsWith("\"") && data[i].endsWith("\""))
				data[i]=data[i].substring(1,data[i].length()-1);
		}
		
		RangeOption option  = null;
		
		int identifier 	= Integer.parseInt(data[0]);
		
		String name = data[1];
		String keyName = data[2];
		String commandName = data[3];
		
		option = new RangeOption(name,keyName);
		
		option.setIdentifier(identifier);
		if(commandName.length()!=0)
			option.setCommandName(commandName);
		
		int optionType = Integer.parseInt(data[4]);
		
		int datapoint1 = 5;
		int datapoint2 = 6;
		
		if(optionType==NO_ARGS_OPTION){	
			option.setType(RangeOption.TYPE.NO_ARGS);
		}
		else if(optionType==SINGLE_STRING_OPTION){
			option.setType(RangeOption.TYPE.SINGLE_STRING);
			option.setString_arg_1(data[datapoint1]);
		}
		else if(optionType==SINGLE_DOUBLE_OPTION){
			option.setType(RangeOption.TYPE.SINGLE_DOUBLE);
			option.setDouble_arg_1(Double.parseDouble(data[datapoint1]));
		}
		else if(optionType==SINGLE_INTEGER_OPTION){
			option.setType(RangeOption.TYPE.SINGLE_INTEGER);
			option.setInteger_arg_1(Integer.parseInt(data[datapoint1]));
		}
		else if(optionType==DUAL_INTEGER_OPTION){
			option.setType(RangeOption.TYPE.DUAL_INTEGER);
			option.setInteger_arg_1(Integer.parseInt(data[datapoint1]));
			option.setInteger_arg_2(Integer.parseInt(data[datapoint2]));
		}
		else if(optionType==SINGLE_SELECT_OPTION){
			option.setType(RangeOption.TYPE.SINGLE_SELECT);
			if(data[datapoint1].startsWith("{") && data[datapoint1].endsWith("}")){
				String strings = data[datapoint1].substring(1,data[datapoint1].length()-1);
				String[] displays = strings.split(",");
				List<String>enums = new ArrayList<String>();
				for(int i=0;i<displays.length;i++){
					enums.add(displays[i]);
				}
				option.setSelect_arg_1(enums);
			}
			else{
				throw new Exception("Incorrect definition of data");
			}
		}
		else if(optionType==DUAL_SELECT_OPTION){
			option.setType(RangeOption.TYPE.DUAL_SELECT);
			if(data[datapoint1].startsWith("{") && data[datapoint1].endsWith("}")){
				String strings = data[datapoint1].substring(1,data[datapoint1].length()-1);
				String[] displays = strings.split(",");
				List<String>enums = new ArrayList<String>();
				for(int i=0;i<displays.length;i++){
					enums.add(displays[i]);
				}
				option.setSelect_arg_1(enums);
			}
			else{
				throw new Exception("Incorrect definition of data");
			}
			if(data[datapoint2].startsWith("{") && data[datapoint2].endsWith("}")){
				String strings = data[datapoint2].substring(1,data[datapoint2].length()-1);
				String[] displays = strings.split(",");
				List<String>enums = new ArrayList<String>();
				for(int i=0;i<displays.length;i++){
					enums.add(displays[i]);
				}
				option.setSelect_arg_2(enums);
			}
			else{
				throw new Exception("Incorrect definition of data");
			}
		}
		
		else{
			throw new Exception("Unknown type of option");
		}
		
		return option;
		
	}
	
	public static List<RangeOption> parseOptions(List<String[]> data){
		
		List<RangeOption> list = new ArrayList<RangeOption>();
		try{
			//creation of the different options
			for(int i=0;i<data.size();i++){
				String [] temp = data.get(i);
				RangeOption ro = createOption(temp);
				list.add(ro);
			}
		}
		catch(Exception exc){
			exc.printStackTrace();
			System.err.println("Configuration file not parsed. Aborting program.");
			System.exit(1);
		}
		
		
		return list;
	}

}
