package var.experimenter;

import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextField;

public class RangeOption  implements ActionListener, Serializable{

	public static final long serialVersionUID = 11568944l;
	
	public enum TYPE{NO_ARGS,SINGLE_STRING,SINGLE_DOUBLE,SINGLE_INTEGER,DUAL_INTEGER,SINGLE_SELECT,DUAL_SELECT};
	private TYPE type = TYPE.NO_ARGS;
	
	
	private String name = "";
	private String keyName = "";
	private String commandName = "";
	
	private int identifier = -1;
	private String string_arg_1		= "";
	private int integer_arg_1	 	= 0;
	private int integer_arg_2 		= 0;
	private double double_arg_1   	= 0;
	private List<String> select_arg_1 = new ArrayList<String>();
	private List<String> select_arg_2 = new ArrayList<String>();
	private boolean isRange1		= false;
	private boolean isRange2		= false;
	private boolean isRange			= false;
	
	private boolean canBeRange		= false;
	private JTextField firstTextField = new JTextField(20);
	private JTextField secondTextField = new JTextField(20);
	private JCheckBox includeOption = new JCheckBox("Include",true);
	private Component parent = null;
	
	private ChoosePanel choosePanel1	= null;
	private ChoosePanel choosePanel2	= null;
	private JPanel panel = new JPanel();
	private JLabel isRangedLabel = null;
	private List<Double> double_values = new ArrayList<Double>();
	private List<Integer> integer_values_1 = new ArrayList<Integer>();
	private List<Integer> integer_values_2 = new ArrayList<Integer>();
	
	private JComboBox combo_1 = null;
	private JComboBox combo_2 = null;
	
	private JButton editButton = null;
	

	public RangeOption(String name,String keyName){
		this.name = name;
		this.keyName = keyName;
	}
	
	
	public JPanel createPanel(Component parent,Dimension d){
		this.parent = parent;
		panel.setLayout(new GridLayout(1,8));
		panel.setMinimumSize(d);
		panel.setMaximumSize(d);
		panel.setPreferredSize(d);
		panel.setBorder(BorderFactory.createEtchedBorder());
		panel.add(includeOption);
		JLabel nameLabel = new JLabel(name);
		panel.add(nameLabel);
		
		switch(type){
		case NO_ARGS:
			panel.add(new JLabel(""));
			panel.add(new JLabel(""));
			panel.add(new JLabel(""));
			panel.add(new JLabel(""));
			canBeRange = false;
			break;
		case SINGLE_STRING:
			firstTextField.setText(string_arg_1);
			panel.add(new JLabel("Parameter (String)"));
			panel.add(firstTextField);
			panel.add(new JLabel(""));
			panel.add(new JLabel(""));
			canBeRange = false;
			break;
		case SINGLE_DOUBLE:
			firstTextField.setText(""+double_arg_1);
			firstTextField.setEditable(false);
			panel.add(new JLabel("Parameter (floating point)"));
			panel.add(firstTextField);
			panel.add(new JLabel(""));
			panel.add(new JLabel(""));
			choosePanel1 = new ChoosePanel(name,new FixedOption(""+double_arg_1),new FixedRangeOption(),new VariableRangeExpOption(),new VariableRangeLinearOption(),0);
			canBeRange = true;
			break;
		case SINGLE_INTEGER:
			firstTextField.setText(""+integer_arg_1);
			firstTextField.setEditable(false);
			panel.add(new JLabel("Parameter (integer)"));
			panel.add(firstTextField);
			panel.add(new JLabel(""));
			panel.add(new JLabel(""));
			choosePanel1 = new ChoosePanel(name,new FixedOption(""+integer_arg_1),new FixedRangeOption(),new VariableRangeExpOption(),new VariableRangeLinearOption(),0);
			canBeRange = true;
			break;
		case DUAL_INTEGER:
			firstTextField.setText(""+integer_arg_1);
			firstTextField.setEditable(false);
			panel.add(new JLabel("1st Parameter (integer)"));
			panel.add(firstTextField);
			secondTextField.setText(""+integer_arg_2);
			secondTextField.setEditable(false);
			panel.add(new JLabel("2nd Parameter (integer)"));
			panel.add(secondTextField);
			canBeRange = true;
			choosePanel1 = new ChoosePanel(name,new FixedOption(""+integer_arg_1),new FixedRangeOption(),new VariableRangeExpOption(),new VariableRangeLinearOption(),0);
			choosePanel2 = new ChoosePanel(name,new FixedOption(""+integer_arg_2),new FixedRangeOption(),new VariableRangeExpOption(),new VariableRangeLinearOption(),0);
			break;
		case SINGLE_SELECT:
			panel.add(new JLabel("1st Parameter(select)"));
			combo_1 = new JComboBox(select_arg_1.toArray());
			panel.add(combo_1);
			panel.add(new JLabel(""));
			panel.add(new JLabel(""));
			canBeRange = false;
			break;
		case DUAL_SELECT:
			panel.add(new JLabel("1st Parameter(select)"));
			combo_1 = new JComboBox(select_arg_1.toArray());
			panel.add(combo_1);
			panel.add(new JLabel("2nd Parameter(select)"));
			combo_2 = new JComboBox(select_arg_2.toArray());
			panel.add(combo_2);
			canBeRange = false;
			break;
		}
		
		isRangedLabel = new JLabel("is range : "+isRange);
		panel.add(isRangedLabel);
		
		editButton = new JButton("Edit range");
		editButton.setEnabled(canBeRange);
		editButton.addActionListener(this);
		panel.add(editButton);
		
		includeOption.addActionListener(this);
		rangeChanged();
		
		return panel;
	}
	
	public boolean mustBeUsed(){
		return includeOption.isSelected();
	}
	
	public void setBeUsed(boolean b){
		includeOption.setSelected(b);
	}
	public void setVisible(boolean b){
		panel.setEnabled(b);
		panel.setVisible(b);
		panel.repaint();
		panel.updateUI();
	}
	
	public String getNoRangeCLOptions(){
		String result = "";
		switch(type){
		case NO_ARGS : break;
		case SINGLE_STRING: 
		case SINGLE_DOUBLE:
		case SINGLE_INTEGER:
			result+=firstTextField.getText();
			break;
		case DUAL_INTEGER:
			result+=firstTextField.getText()+" ";
			result+=secondTextField.getText();
			break;
		case SINGLE_SELECT : 
			int temp = combo_1.getSelectedIndex();
			result+=select_arg_1.get(temp);
			break;
		case DUAL_SELECT:
			int te1 = combo_1.getSelectedIndex();
			result+=select_arg_1.get(te1)+" ";
			int te2 = combo_2.getSelectedIndex();
			result+=select_arg_2.get(te2);
			break;
		}
		return result;
	}
	
	public List<String> getRangeCLOptions(){
		List<String> result = new ArrayList<String>();
		switch(type){
		case NO_ARGS : break; //actually impossible
		case SINGLE_STRING: break;	//actually impossible
		case SINGLE_DOUBLE : 
			for(int i=0;i<double_values.size();i++){
				result.add(double_values.get(i)+"");
			}
			break;
		case SINGLE_INTEGER: 
			for(int i=0;i<integer_values_1.size();i++){
				result.add(integer_values_1.get(i)+"");
			}
			break;
		case DUAL_INTEGER:
			for(int i=0;i<integer_values_1.size();i++){
				for(int j=0;j<integer_values_2.size();j++){
					result.add(integer_values_1.get(i)+" "+integer_values_2.get(j));
				}
			}
			break;
		case SINGLE_SELECT : break;
		case DUAL_SELECT : break;
		}
		
		return result;
	}
	
	private void rangeChanged(){
		//set label
		Color col = null;
		if(includeOption.isSelected()){
			if(isRange)
				col = Color.BLUE;
			else if(!isRange && canBeRange)
				col = Color.black;
			else
				col = Color.LIGHT_GRAY;
		}
		else{
			col = Color.magenta;
		}
		
		
		try{
			isRangedLabel.setForeground(col);
			boolean inc = includeOption.isSelected();
		
			if(canBeRange && inc)
				isRangedLabel.setText("is range : "+isRange);
	
			if(!canBeRange && inc)
				isRangedLabel.setText("No Range");
			if(!includeOption.isSelected())
				isRangedLabel.setText("Not included");
		
			
			//		change content of textfield(s)
			if(type==TYPE.SINGLE_DOUBLE){
				double_values = choosePanel1.getDoubleValues();
				setTextFieldText(firstTextField,double_values);
			}
			else if(type==TYPE.SINGLE_INTEGER){
				integer_values_1 = choosePanel1.getIntegerValues();
				setTextFieldText(firstTextField,integer_values_1);
			}
			else if(type==TYPE.DUAL_INTEGER){
				integer_values_1 = choosePanel1.getIntegerValues();
				integer_values_2 = choosePanel2.getIntegerValues();
				setTextFieldText(firstTextField,integer_values_1);
				setTextFieldText(secondTextField,integer_values_2);
			}
		}
		catch(NullPointerException exc){
			//startup problems
		}
		panel.repaint();
	}
	
	public void actionPerformed(ActionEvent e){
		if(e==null)
			return;
		if(e.getSource().equals(editButton)){
			if(type!=TYPE.DUAL_INTEGER){
				JOptionPane.showMessageDialog(parent,choosePanel1,"Please edit range",JOptionPane.INFORMATION_MESSAGE);
				isRange = choosePanel1.isRange();
				rangeChanged();
			}
			else{
				JOptionPane.showMessageDialog(parent,choosePanel1,"Please edit range param 1",JOptionPane.INFORMATION_MESSAGE);
				JOptionPane.showMessageDialog(parent,choosePanel2,"Please edit range param 2",JOptionPane.INFORMATION_MESSAGE);
				isRange1= choosePanel1.isRange();
				isRange2= choosePanel2.isRange();
				isRange =isRange1 || isRange2;
				rangeChanged();
			}		
		}
		
		else if(e.getSource().equals(includeOption)){

		}
	}
	
	
	public JPanel getPanel(){
		return panel;
	}
	
	private void setTextFieldText(JTextField field,List<? extends Number> list){
		if(list.size()==1)
			field.setText(""+list.get(0));
		else if(list.size()>1)
			field.setText(list.get(0)+" UP TO "+list.get(list.size()-1));
	}
	
	public String toString(){
		String result = name+" "+keyName+" ";
		switch(type){
		case NO_ARGS:break;
		case SINGLE_STRING: result+=string_arg_1; break;
		case SINGLE_DOUBLE: result+=double_arg_1; break;
		case SINGLE_INTEGER: result+=integer_arg_1;break;
		case DUAL_INTEGER: result+=integer_arg_1+" "+integer_arg_2;break;
		case SINGLE_SELECT :result+=select_arg_1.get(combo_1.getSelectedIndex());break;
		case DUAL_SELECT: result+=select_arg_1.get(combo_1.getSelectedIndex())+" ";
							result+=select_arg_2.get(combo_2.getSelectedIndex()); break;
		default : break;
				
		}
		return result;
	}
	
	
	/*
	 * ===============================================================
	 * Getters and setters
	 * ===============================================================
	 */
	
	public int getIdentifier() {
		return identifier;
	}
	public void setIdentifier(int identifier) {
		this.identifier = identifier;
	}
	public String getName() {
		return name;
	}
	public void setName(String name) {
		this.name = name;
	}
	public TYPE getType() {
		return type;
	}
	public void setType(TYPE type) {
		this.type = type;
	}
	public String getKeyName() {
		return keyName;
	}
	public void setKeyName(String keyName) {
		this.keyName = keyName;
	}
	public double getDouble_arg_1() {
		return double_arg_1;
	}
	public void setDouble_arg_1(double double_arg) {
		this.double_arg_1 = double_arg;
	}
	public int getInteger_arg_1() {
		return integer_arg_1;
	}
	public void setInteger_arg_1(int integer_arg_1) {
		this.integer_arg_1 = integer_arg_1;
	}
	public int getInteger_arg_2() {
		return integer_arg_2;
	}
	public void setInteger_arg_2(int integer_arg_2) {
		this.integer_arg_2 = integer_arg_2;
	}
	public List<String> getSelect_arg_1() {
		return select_arg_1;
	}
	public void setSelect_arg_1(List<String> select_arg_1) {
		this.select_arg_1 = select_arg_1;
	}
	public List<String> getSelect_arg_2() {
		return select_arg_2;
	}
	public void setSelect_arg_2(List<String> select_arg_2) {
		this.select_arg_2 = select_arg_2;
	}
	public String getString_arg_1() {
		return string_arg_1;
	}
	public void setString_arg_1(String string_arg_1) {
		this.string_arg_1 = string_arg_1;
	}
	public String getCommandName() {
		return commandName;
	}
	public void setCommandName(String commandName) {
		this.commandName = commandName;
	}
}
