/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
package util.parsers.fasta;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileFilter;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

public class ExtractTrainingData {

	public static final String DATADIRECTORY = "./origdata/";
	public static final int MAX_EXTRACT = 150;
	public static boolean PURE_DNA = false;
	
	public static void main(String[] args){
		if(args.length==0)
			new ExtractTrainingData(DATADIRECTORY);
		else
			new ExtractTrainingData(args);
	}
	
	private class Filter implements FileFilter{
		public boolean accept(File file){
			if(file.getName().endsWith(".embl"))
				return true;
			else
				return false;
		}
	}
	
	private class AnnotationData{
		public String sequence = null;
		public boolean complement = false;
		public boolean hasFault = false;
		public List<Integer> donorsites = new ArrayList<Integer>();
		public List<Integer> acceptorsites = new ArrayList<Integer>();
	}
	
	public ExtractTrainingData(String dir){
		try{
			File file = new File(dir);
			FileFilter filter = new Filter();
			File[] files = file.listFiles(filter);
			String[] s = new String[files.length];
			for(int i=0;i<files.length;i++){
				s[i] = files[i].getAbsolutePath();
			}
			new ExtractTrainingData(s);
		}
		catch(Exception exc){
			exc.printStackTrace();
		}
	}
	
	public ExtractTrainingData(String[] args){
		try{
			BufferedWriter posDataAcc = new BufferedWriter(new FileWriter(DATADIRECTORY+"acceptors.pos"));
			BufferedWriter posDataDon = new BufferedWriter(new FileWriter(DATADIRECTORY+"donors.pos"));
			BufferedWriter negDataAcc = new BufferedWriter(new FileWriter(DATADIRECTORY+"acceptors.neg"));
			BufferedWriter negDataDon = new BufferedWriter(new FileWriter(DATADIRECTORY+"donors.neg"));	
			for(int i=0;i<args.length;i++){
				System.out.println("--> "+args[i]);
				AnnotationData data = readData(args[i]);
				if(!data.hasFault){
					if(data.complement)
						reverse(data);
					writeAcceptors(posDataAcc,negDataAcc,data);
					writeDonors(posDataDon,negDataDon,data);
				}
			}
			posDataAcc.close();
			posDataDon.close();
			negDataAcc.close();
			negDataDon.close();
		}
		catch(Exception exc){
			exc.printStackTrace();
		}
	}
	
	public void reverse(AnnotationData data){
		String sequence = data.sequence;
		StringBuffer newSequence = new StringBuffer();
		for(int i=0;i<sequence.length();i++){
			if(sequence.charAt(i)=='A')
				newSequence.append('T');
			else if(sequence.charAt(i)=='T')
				newSequence.append('A');
			else if(sequence.charAt(i)=='G')
				newSequence.append('C');
			else if(sequence.charAt(i)=='C')
				newSequence.append('G');
		}
		data.sequence = new String(newSequence);
	}
	
	public void writeAcceptors(BufferedWriter posData,BufferedWriter negData,AnnotationData data) throws IOException{
		for(int i=0;i<data.sequence.length()-2;i++){
			if(data.sequence.substring(i,i+2).equals("AG")){
				String s = extractFromSequence(i,data.sequence);
				if(!data.acceptorsites.contains(new Integer(i))){
					if(PURE_DNA){
						if(!s.contains("N"))
							negData.write(s+"\n");
					}
					else{
						negData.write(s+"\n");
					}
				}
			}
		}
		for(int i=0;i<data.acceptorsites.size();i++){
			String s = extractFromSequence(data.acceptorsites.get(i),data.sequence);
			if(PURE_DNA){
				if(!s.contains("N"))
					posData.write(s+"\n");
			}
			else{
				posData.write(s+"\n");
			}
			
		}
	}
	
	public void writeDonors(BufferedWriter posData,BufferedWriter negData,AnnotationData data) throws IOException{
		for(int i=0;i<data.sequence.length()-2;i++){
			if(data.sequence.substring(i,i+2).equals("GT")){
				String s = extractFromSequence(i,data.sequence);
				if(!data.donorsites.contains(new Integer(i))){
					if(PURE_DNA){
						if(!s.contains("N"))
							negData.write(s+"\n");
					}
					else{
						negData.write(s+"\n");
					}
				}
			}
		}
		for(int i=0;i<data.donorsites.size();i++){
			String s = extractFromSequence(data.donorsites.get(i),data.sequence);
			if(PURE_DNA){
				if(!s.contains("N"))
					posData.write(s+"\n");
			}
			else{
				posData.write(s+"\n");
			}
		}
	}
	
	public String extractFromSequence(int i,String seq){
		if(i>=MAX_EXTRACT && i+MAX_EXTRACT < seq.length())
			return seq.substring(i-MAX_EXTRACT,i+MAX_EXTRACT);
		else if(i>=MAX_EXTRACT && i+MAX_EXTRACT>=seq.length()){
			String temp = seq.substring(i-MAX_EXTRACT,seq.length());
			for(int j=0;j<(i+MAX_EXTRACT-seq.length());j++){
				temp+='N';
			}
			return temp;
		}
		else if(i<MAX_EXTRACT && i+MAX_EXTRACT<seq.length()){
			String temp = seq.substring(0,i+MAX_EXTRACT);
			String temp2 = "";
			for(int j=0;j<(MAX_EXTRACT-i);j++){
				temp2+='N';
			}
			temp = temp2+temp;
			return temp;
		}
		else{
			String temp = seq.substring(0,seq.length());
			String temp2 = "";
			for(int j=0;j<(MAX_EXTRACT-i);j++){
				temp2+='N';
			}
			temp = temp2+temp;
			for(int j=0;j<(i+MAX_EXTRACT-seq.length());j++){
				temp+='N';
			}
			return temp;
		}
	}
	
	public AnnotationData readData(String fileName){
		AnnotationData data = new AnnotationData();
		try{
			BufferedReader reader = new BufferedReader(new FileReader(new File(fileName)));
			String s = reader.readLine();
			parseFirstLine(data,s);
			boolean startSeq = false;
			String sequence = "";
			while(s!=null){
				if(startSeq){
					StringBuffer buffer = new StringBuffer();
					for(int i=0;i<s.length();i++){
						if(s.charAt(i)=='a')
							buffer.append('A');
						else if(s.charAt(i)=='c')
							buffer.append('C');
						else if(s.charAt(i)=='t')
							buffer.append('T');
						else if(s.charAt(i)=='g')
							buffer.append('G');
					}
					sequence+=new String(buffer);
				}
				if(s.startsWith("SQ")||s.startsWith(">"))
					startSeq = true;
				s = reader.readLine();
			}
			data.sequence = sequence;
			reader.close();
		}
		catch(Exception exc){
			exc.printStackTrace();
		}
		return data;
	}
	
	public void parseFirstLine(AnnotationData data,String s){
		
		if(s.indexOf("complement")!=-1)
			data.complement = true;
		else
			data.complement = false;
		
		int t = s.indexOf("join");
		if(t==-1 || s.contains(">")){
			data.hasFault = true;
			return;
		}
			
		
		String p = s.substring(t+5,s.length());
		StringBuffer temp = new StringBuffer();
		for(int i=0;i<p.length();i++){
			
			if(p.charAt(i)=='.'){
				Integer loc = Integer.parseInt(new String(temp));
				loc = loc-3;
				data.acceptorsites.add(loc);
				temp = new StringBuffer();
				i++;
			}
			else if(p.charAt(i)==','||p.charAt(i)==')'){
				Integer loc = Integer.parseInt(new String(temp));
				data.donorsites.add(loc);
				temp = new StringBuffer();
				if(p.charAt(i)==')')
					break;
			}
			else{
				temp.append(p.charAt(i));
			}
		}
	}
	
	
	
	
}
