/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
package util.parsers.fasta;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.util.ArrayList;
import java.util.List;

public class PrintData {

	
	private boolean reverse = false;
	protected List<GeneData> dataList = new ArrayList<GeneData>(); 
	
	public static void main(String[] args){
		new PrintData(args[0]);
	}
	
	//public int[] counters = new int[1000];
	
	public PrintData(String directory){
		File dir = new File(directory);
		if(!dir.exists() || !dir.isDirectory())
			return;
		File [] files = dir.listFiles();
		File result = new File("Results_"+dir.getName()+".txt");
		try{
			BufferedWriter writer = new BufferedWriter(new FileWriter(result));
			
			int correctParsed = 0;
			int incorrectParsed = 0;
			List<String> incorrectList = new ArrayList<String>();
			for(int i=0;i<files.length;i++){
				boolean b = extractDataFasta(files[i],writer);
				if(b)
					correctParsed++;
				else{
					incorrectParsed++;
					incorrectList.add(files[i].getName());
				}
			}

			int num_mul3 = 0;
			int num_mul3_1 = 0;
			int num_mul3_2 = 0;
			
			int correct_start = 0;
			int false_start = 0;
			int correct_stop = 0;
			int false_stop = 0;
			
			for(int i=0;i<dataList.size();i++){
				GeneData dat = dataList.get(i);
				int mul3 = dat.cdsLength;
				if(mul3%3==0) num_mul3++;
				else if(mul3%3==1) num_mul3_1++;
				else if(mul3%3==2) num_mul3_2++;
				
				if(dat.correctStartCodon)correct_start++;
				else false_start++;
			
				if(dat.correctStopCodon)correct_stop++;
				else false_stop++;
			}
			writer.write("\n\n\n\n\n");
			writer.write("STATISTICS \n");
			writer.write("======================================== \n");
			writer.write("======================================== \n");
			writer.write("\n");
			
			writer.write("Stats for directory :"+dir.getName());writer.write("\n");
			writer.write("Total number of files : "+dataList.size());writer.write("\n");
			writer.write("# of correct parsed files : " + correctParsed);writer.write("\n");
			writer.write("# of incorrect parsed files : "+ incorrectParsed);writer.write("\n");
			for(int i=0;i<incorrectList.size();i++){
				writer.write("\t\t -> "+incorrectList.get(i));writer.write("\n");
			}
			writer.write("");writer.write("\n");
			writer.write("# of correct cds-length (divisible by 3): "+num_mul3);writer.write("\n");
			writer.write("# of incorrect cds-length (non divisible by 3) : "+(num_mul3_1+num_mul3_2));writer.write("\n");
			writer.write("---> frameshift 1 : "+num_mul3_1);writer.write("\n");
			writer.write("---> frameshift 2 : "+num_mul3_2);writer.write("\n");
			writer.write("");writer.write("\n");
			writer.write("# of correct start codons : "+correct_start);writer.write("\n");
			writer.write("# of incorrect start codons : "+false_start);writer.write("\n");
			writer.write("");writer.write("\n");
			writer.write("# of correct stop codons : "+correct_stop);writer.write("\n");
			writer.write("# of incorrect stop codons : "+false_stop);writer.write("\n");
			writer.write("\n");
			
			/*writer.write("\n");
			for(int i=0;i<counters.length;i++){
				
				int primfN = util.Mathematics.getNumberOfPrimeFactors(i);
				if(primfN==1)
				//if(counters[i]!=0){
					writer.write(i+"\t"+counters[i]+"\n");
				//}
			}
			*/
			
			writer.flush();
			writer.close();
			
			
		}
		catch(Exception exc){
			exc.printStackTrace();
		}
	}
	
	public boolean extractDataEnsembl(File file, BufferedWriter writer){
		try{
			BufferedReader reader = new BufferedReader(new FileReader(file));
			
			List<GeneData> list_data = new ArrayList<GeneData>();
			int basePairs = 0;
			String s = reader.readLine();
			String sequence = "";
			while(s!=null){
				if(s.startsWith("FT")){
					if(s.indexOf("CDS")!=-1){
						GeneData data = new GeneData();
						List<Integer> res = new ArrayList<Integer>();
						if(s.indexOf("complement")==-1)
							data.reverse = false;
						else
							data.reverse = true;
						
						int firstIndex = s.indexOf("join")+5;
						if(s.indexOf("join")==-1)
							firstIndex = s.indexOf("(")+1;
						int temp = 0;
						String t = "";
						for(int i=firstIndex;i<s.length();i++){
							char c = s.charAt(i);
							
							if(c==')'){
								temp = Integer.parseInt(t);
								res.add(temp);
								break;
							}
							else if(c==','){
								temp = Integer.parseInt(t);
								res.add(temp);
								t = "";
							}
							else if(c=='.'){
								temp = Integer.parseInt(t);
								res.add(temp);
								t = "";
								i++;
							}
							else 
								t+=c;
						}
						data.gd_splicesites = res;
						list_data.add(data);
					}
				}
				else if(s.startsWith("SQ")){
					String sub = s.substring(s.indexOf("Sequence")+9,s.indexOf("BP")-1);
					basePairs = Integer.parseInt(sub);
					basePairs++;
				}
				else{
					s = s.trim();
					String [] split = s.split(" ");
					for(int i=0;i<split.length;i++){
						if(split[i].length()==10)
							sequence+=split[i];
						else{
							if(split[i].length()!=1 && !split[i].contains("0")
									&& !split[i].contains("")&& !split[i].contains("2")
									&& !split[i].contains("3")&& !split[i].contains("4")
									&& !split[i].contains("5")&& !split[i].contains("6")
									&& !split[i].contains("7")&& !split[i].contains("8")
									&& !split[i].contains("9")){
								sequence+=split[i];
							}
						}
					}
				}
				s = reader.readLine();
			}
			
			return true;
		}
		catch(Exception exc){
			exc.printStackTrace();
			return false;
		}
	}
	
	
	public boolean extractDataFasta(File file,BufferedWriter writer){
		try{
			GeneData data = new GeneData();
			
			BufferedReader reader = new BufferedReader(new FileReader(file));
			String s = reader.readLine();
			List<Integer> list = parseHeaderFasta(s);
			
			writer.write("DATA FOR "+file.getName()+"\n");
			writer.write("--------------------------------\n");
			writer.write("-->Number of indicated exons :"+list.size()/2+"\n");
			s = reader.readLine(); //empty >
			s = reader.readLine();
			String sequence = "";
			
			while(s!=null){
				s = s.trim();
				sequence+=s;
				s = reader.readLine();
			}
			
			if(reverse)
				writer.write("-->reverse strand \n");
			else
				writer.write("-->forward strand \n");
			

			data.reverse = reverse;
			data.numExons = list.size()/2;
			data.name = file.getName();
			data.gd_splicesites = list;
			
			if(!reverse){
				
				String startWithATG = checkATG(sequence,list.get(0));
				int mul3 = checkMul3(list);
				String endWithStop = checkStop(sequence,list.get(list.size()-1));		
				boolean c_startWithATG = startWithATG.equals("ATG");
				boolean c_mul3 = (mul3%3==0);
				boolean c_endWithStop = endWithStop.equals("TAG")||endWithStop.equals("TGA")||endWithStop.equals("TAA");
				writer.write("Correct startcodon (ATG) : "+c_startWithATG+" | startcodon : "+startWithATG+"\n");
				writer.write("CDS length multiple of 3 : "+c_mul3+" | cds-length : "+mul3+" \n");
				writer.write("Stopcodon present? (TAG|TGA|TAA) : "+c_endWithStop+" | stopcodon : "+endWithStop+" \n");
				
				/*int[] numPrim = util.Mathematics.getPrimeFactors(mul3);
				writer.write("Number of cds prime factors : "+util.Mathematics.getNumberOfPrimeFactors(mul3)+"\n");
				for(int i=0;i<numPrim.length;i++){
					writer.write(numPrim[i]+" ");
				}
				writer.write("\n");*/
				
				writer.write(" \n");
				writer.write("intron/exon-boundaries :: \n");
				
				data.correctStartCodon = c_startWithATG;
				data.correctStopCodon = c_endWithStop;
				data.correctCdsLength = c_mul3;
				data.startCodon = startWithATG;
				data.stopCodon = endWithStop;
				data.cdsLength = mul3;
				List<String> bounds = new ArrayList<String>();
				
				for(int i=1;i<list.size()-1;i++){
					String dinuc = "";
					int splice = list.get(i);
					if(i%2==0){
						dinuc = sequence.substring(splice-3,splice-1);
						writer.write(splice +" : acceptor (should be AG) : "+dinuc+"\n");
					}
					else{
						dinuc = sequence.substring(splice,splice+2);
						writer.write(splice+" : donor (should be GT/GC)  : "+dinuc+"\n");
					}
					bounds.add(dinuc);
				}
				data.boundaries = bounds;
				dataList.add(data);
			}
			else{
				String startWithCAT = checkCAT(sequence,list.get(list.size()-1));
				int mul3 = checkMul3(list);
				String endWithStop = checkStopReverse(sequence,list.get(0));				
				boolean c_startWithCAT =  startWithCAT.equals("CAT");
				boolean c_mul3 = (mul3%3==0);
				boolean c_endWithStop = endWithStop.equals("CTA")||endWithStop.equals("TCA")||endWithStop.equals("TTA");
				writer.write("Correct startcodon (CAT) : "+c_startWithCAT+" | startcodon : "+startWithCAT+"\n");
				writer.write("CDS length multiple of 3 : "+c_mul3+" | cds-length : "+mul3+" \n");
				writer.write("Stopcodon present? (CTA|TCA|TTA) : "+c_endWithStop+" | stopcodon : "+endWithStop+" \n");
				
			/*	int[] numPrim = util.Mathematics.getPrimeFactors(mul3);
				writer.write("Number of cds prime factors : "+util.Mathematics.getNumberOfPrimeFactors(mul3)+"\n");
				for(int i=0;i<numPrim.length;i++){
					writer.write(numPrim[i]+" ");
				}
				writer.write("\n");*/
				
				writer.write(" \n");
				writer.write("intron/exon-boundaries :: \n");
				
				data.correctStartCodon = c_startWithCAT;
				data.correctStopCodon = c_endWithStop;
				data.correctCdsLength = c_mul3;
				data.startCodon = startWithCAT;
				data.stopCodon = endWithStop;
				data.cdsLength = mul3;
				List<String> bounds = new ArrayList<String>();
				
				for(int i=1;i<list.size()-1;i++){
					String dinuc = "";
					int splice = list.get(i);
					if(i%2==0){
						dinuc = sequence.substring(splice-3,splice-1);
					//	dinuc = Util.toForwardStrand(dinuc);
						writer.write(splice+" : donor (should be AC/GC)   : "+dinuc+"\n");
					}
					else{
						dinuc = sequence.substring(splice,splice+2);
					//	dinuc = Util.toForwardStrand(dinuc);
						writer.write(splice +" : acceptor (should be CT)  : "+dinuc+"\n");
					}
					bounds.add(dinuc);
				}
				
				
				data.boundaries = bounds;
				dataList.add(data);
			}
			
		/*	int [][] prims = getPrimes(list);
			for(int i=0;i<prims.length;i++){
				int[] t = prims[i];
				for(int j=0;j<t.length;j++){
					writer.write(t[j]+" ");
					counters[t[j]]++;
				}
				writer.write("\n");
			}*/
			
			writer.write("\n\n\n\n");
			writer.flush();
			reader.close();
			
			
			return true;
		}
		catch(Exception exc){
			System.err.println("Error occured while processing file : "+file.getName());
			//exc.printStackTrace();
			return false;
		}
	}
	
	//ALL
	public int checkMul3(List<Integer> list){
		int length = 0;
		for(int i = 0;i<list.size();i+=2){
			int temp = list.get(i+1)-list.get(i)+1;
			length+=temp;
		}
		return length;
	}
	
	//FORWARD
	public String checkStop(String sequence,int last){
		String temp = sequence.substring(last-3,last);
		return temp;
	}
	
	public String checkATG(String sequence,int start){
		String temp  = sequence.substring(start-1,start+2);
		return temp;
	}
	
	
	//REVERSE
	public String checkStopReverse(String sequence,int last){
		String temp = sequence.substring(last-1,last+2);
		return temp;
	}
	
	public String checkCAT(String sequence,int start){
		String temp = sequence.substring(start-3,start);
		return temp;
	}
	
	
	
	public List<Integer> parseHeaderFasta(String s){
		List<Integer> res = new ArrayList<Integer>();
		if(s.indexOf("complement")!=-1)
			reverse = true;
		else
			reverse = false;
		
		int firstIndex = s.indexOf("join")+5;
		if(s.indexOf("join")==-1)
			firstIndex = s.indexOf("(")+1;
		int temp = 0;
		String t = "";
		for(int i=firstIndex;i<s.length();i++){
			char c = s.charAt(i);
			
			if(c==')'){
				temp = Integer.parseInt(t);
				res.add(temp);
				break;
			}
			else if(c==','){
				temp = Integer.parseInt(t);
				res.add(temp);
				t = "";
			}
			else if(c=='.'){
				temp = Integer.parseInt(t);
				res.add(temp);
				t = "";
				i++;
			}
			else 
				t+=c;
		}
		return res;
	}
	
	private class GeneData{
		public String name = "";
		
		public int numExons = 0;
		public boolean reverse = false;
		
		public String startCodon = "";
		public boolean correctStartCodon = false;
		
		public String stopCodon = "";
		public boolean correctStopCodon = false;
		
		public int cdsLength = 0;
		public boolean correctCdsLength = false;
		
		public List<Integer> gd_splicesites = new ArrayList<Integer>();
		public List<String> boundaries = new ArrayList<String>();
		
	}
	
}
